///
///	@file 	gaCompatModule.cpp
/// @brief 	Compatibility module for the GoAhead Web Server
///	@overview Provide a reasonable measure of compatibility with the 
///		GoAhead WebServer APIs.
///	@remarks This module only supports single threaded operation without 
///	any support for virtual hosts.
//
/////////////////////////////////// Copyright //////////////////////////////////
//
//	@copy	default
//	
//	Copyright (c) Embedthis Software LLC, 2003-2009. All Rights Reserved.
//	
//	This software is distributed under commercial and open source licenses.
//	You may use the GPL open source license described below or you may acquire 
//	a commercial license from Embedthis Software. You agree to be fully bound 
//	by the terms of either license. Consult the LICENSE.TXT distributed with 
//	this software for full details.
//	
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version. See the GNU General Public License for more 
//	details at: http://www.mbedthis.com/downloads/gplLicense.html
//	
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	
//	This GPL license does NOT permit incorporating this software into 
//	proprietary programs. If you are unable to comply with the GPL, you must
//	acquire a commercial license to use this software. Commercial licenses 
//	for this software and support services are available from Embedthis 
//	Software at http://www.mbedthis.com 
//	
//	@end
//
////////////////////////////////// Includes ////////////////////////////////////

#include	"gaCompatModule.h"

#include "mtd/mtd-user.h"

//////////////////////////////////// Locals ////////////////////////////////////
#if BLD_FEATURE_GACOMPAT_MODULE

#define MPR_HTTP_MAX_GO_FORM		100
//
//	hAlloc list defines.
//
//	The handle list stores the length of the list and the number of used
//	handles in the first two words.  These are hidden from the caller by
//	returning a pointer to the third word to the caller
//
#define H_LEN		0		// First entry holds length of list
#define H_USED		1		// Second entry holds number of used
#define H_OFFSET	2		// Offset to real start of list 
#define H_INCR		16		// Grow handle list in chunks this size

/* ASP KB config define*/
#define CONFIGSYSTEM 1
#define DISPLAYSYSMODE 2
#define DISPLAYSYSLAN 3
#define DISPLAYSYSBG 4
#define DISPLAYVER 5
#define DISPLAYBACKLIGHT 6
#define DISPLAYMODEADDR 7
#define DISPLAYPORT 8
#define DISPLAYJOYSTICKSPEED 9
#define DISPLAYSERIALPORTBAUDRATE 10
#define DISPLAYSERIALPORTMODE 11
#define DISPLAYIPADDR 12
#define DISPLAYVBID 13
#define DISPLAYVBPORT 14
#define DISPLAYVBIPADDR 15
#define DISPLAYSYSNETWORKCONFIG 16
#define DISPLAYBAUDRATE 17
#define REBOOT 18
#define SWITCHTOFRAME 19
#define JUMP 20
#define SETMAXPROMODEADDR 21
#define SETMAXPROIPADDR 22
#define SETMAXPROPORT 23 
#define SETMAXPROJOYSTICKSPEED 24
#define SETMAXPROSERIALPORTBAUDRATE 25
#define SETMAXPROSERIALPORTMODE 26
#define SETVBID 27
#define SETVBBAUDRATE 28
#define SETVBIPADDR 29
#define SETVBPORT 30
#define SETSYSIPADDR 31
#define SETSYSNETMASK 32
#define SETSYSGATEWAY 33
#define SETSYSDNS 34
#define  DISPLAYHARDKEYACTION 35
#define  DISPLAYHARDKEYACTIONINDEX 36
#define  DISPLAYACTION 37
#define  DISPLAYHARDKEYNAME 38
#define   FORMHARDKEYCONFIG  39
#define   FORMCHANGEHARDKEY     40

#define SERVER_IP "127.0.0.1"
#define SERVER_PORT 2280
#define MAXMESSAGELEN 256
#define VERSION_STR_SIZ	32
#define FPGA_DEVICE "/dev/mtd6"
#define APPLICATION_DEVICE "/dev/mtd8"
#define KERNEL_DEVICE "/dev/mtd4"
#define FS_DEVICE "/dev/mtd5"
#define REBOOTFLAG 12333
#define FILE_APPLICATION 1
#define FILE_FPGA 2
#define FILE_KERNEL 3
#define FILE_SILDERFW 4
#define FILE_FIRMWARE_ULTRAKEYPLUS 5
#define FILE_FIRMWARE_CLIENT 6
#define FILE_DYNAMIC_BUTTON 7
#define FILE_FS 8
#define FILE_BL 9
#define UPLOAD_STATUS_CONNTINUE 0
#define UPLOAD_STATUS_ERROR 2
#define UPLOAD_STATUS_OK 3
#define UPLOAD_STATUS_END 5
#define UPLOAD_PROCESS_TYPE_ERASE 1
#define UPLOAD_PROCESS_TYPE_WRITE 2
#define UPLOAD_PROCESS_TYPE_WAIT  3
#define ETHERNET_IP_LEN			4
#define ETHERNET_NETMSK_LEN		4
#define ETHERNET_GATEWAY_LEN		4
#define ETHERNET_DNS_LEN			4

static int changePassworkError = 0;
static int uploadStatus = 0;
static int upload_type = 0;
static int file_length = 0;
static int total_length = 0;
static int upload_length = 0;
static int folderType = -1;
char tmpFile[] = "/tmp/uploadx";
static int SladerWait = 0;
static int  hardKey_index =1;   //default __KEY_F1
char *globalPostData = NULL;
int globalPostLen = 0;

int PrimaryUDP;
struct sockaddr_in ServerSock;
char sendbuf[MAXMESSAGELEN];
char recvbuf[MAXMESSAGELEN];

int mksock(int type)
{
    int sock = socket(AF_INET, type, 0);
    if (sock < 0)
    {
        printf("create socket error");
	 return -1;
    }
    return sock;
}

void BindSock(int sock)
{
    struct sockaddr_in sin;
    bzero(&sin,sizeof(struct sockaddr_in));  
    sin.sin_family = AF_INET;   
    sin.sin_addr.s_addr = INADDR_ANY;  
    sin.sin_port = 0; 
    
    bind(sock, (struct sockaddr*)&sin, sizeof(sin));
}

///////////////////web based/////////////////////////

void websSetVar(webs_t wp, char *var, char *value)
{
	MaRequest	*rq;

	rq = (MaRequest*) wp;
	rq->setVar(MA_GLOBAL_OBJ, var, value);
}

char *websGetVar(webs_t wp, char *var, char *def)
{
	MaRequest	*rq;
	char		*value;

	rq = (MaRequest*) wp;

	if ((value = rq->getVar(MA_FORM_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_HEADERS_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_REQUEST_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_SERVER_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_GLOBAL_OBJ, var, 0))  != 0) {
		return value;
	}
	return def;
}


typedef int BOOL;

typedef enum {	SB1200, SB1800, SB2400, SB4800, SB9600, SB19200, SB57600, SB_last 	} SelectableSerialBaudRate_t;
typedef enum {	DB7, DB8, DB_last	} SelectableSerialDataBit_t;
typedef enum {	Podd, Peven, Pnone, P_last	} SelectableSerialParity_t;
typedef enum {	StopBit1, StopBit2, StopBit_last	}	SelectableSerialStopBit_t;
//typedef enum {	SD

//////////////////structures/////////////////////////
typedef enum {	Mode_Standalone,
				Mode_VideBLX,
				Mode_Maxpro,
				Mode_Last,
} UltraKeyLite_Mode_t;

typedef enum {
	KPB_Off,
	KPB_Low,
	KPB_Middle,
	KPB_High,
	KPB_Last,
} KeyPad_Brightness_t;

typedef struct EtherNetwork * PtrEtherNetwork_t;

typedef struct EtherNetwork 
{
	unsigned char IP[ETHERNET_IP_LEN];
	unsigned char NetMsk[ETHERNET_NETMSK_LEN];
	unsigned char GateWay[ETHERNET_GATEWAY_LEN];
	unsigned char DNS[ETHERNET_DNS_LEN];
} EtherNetwork_t;

typedef struct List * ptrList_t;

typedef struct List {
	ptrList_t next;
	ptrList_t prev;
} List_t;

typedef struct CallBackArgs * ptr_CallBackArgs_t;

typedef struct CallBackArgs {
	unsigned int num;
} CallBackArgs_t;

typedef struct CallBackAction {
	List_t list; 
	ptr_CallBackArgs_t arg;
	unsigned int func_index;
} CallBackAction_t;

#define MAX_DYN_KEY_NUM	0xFE

typedef struct SysKeyCallBackObject {
	BOOL dirty;
	List_t SysKeyCallBackList[MAX_DYN_KEY_NUM];
} SysKeyCallBackObject_t;

SysKeyCallBackObject_t SysKeyCallBackListObj;

void Switch_to_frame()
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf,"%c",SWITCHTOFRAME);
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	

}


KeyPad_Brightness_t GetKeyPadBackLightBright(void)
{
        int keypad_backbrightness;
        struct timeval tv;
        fd_set readfds;
        tv.tv_sec = 1;
        tv.tv_usec = 0;
        FD_ZERO(&readfds);
        FD_SET(PrimaryUDP, &readfds);

        memset(sendbuf,0,sizeof(sendbuf));
        memset(recvbuf,0,sizeof(recvbuf));
        socklen_t fromlen = sizeof(ServerSock);
        sendbuf[0] = DISPLAYBACKLIGHT;
        sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
        if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
        recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
	keypad_backbrightness = recvbuf[0];
        if(keypad_backbrightness < 10)
	{
		return KPB_Off;
	}
	else if(keypad_backbrightness < 40)
	{
		return KPB_Low;
	}
	else if(keypad_backbrightness < 150)
	{
		return KPB_Middle;
	}
	else
		return KPB_High;
}


static int aspDisplaySystemConfigSysMode(webs_t wp)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYSYSMODE;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
        recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    //printf("DISPLAYSYSMODE=%d\n",recvbuf[0]);
    switch(recvbuf[0])
    {
        case Mode_VideBLX:
            return websWrite(wp, T("%d"), 1);
            break;
        case Mode_Maxpro:
            return websWrite(wp, T("%d"), 2);
            break;
        default :
            return websWrite(wp, T(""));
            break;
    }
    return websWrite(wp, T(""));
}


static int aspDisplaySystemConfigBackLight(webs_t wp)
{
	KeyPad_Brightness_t back = KPB_Last;

	back = GetKeyPadBackLightBright();

	switch(back)
	{
		case KPB_Off:
			return websWrite(wp, T("%d"), 0);
			break;
		case KPB_Low:
			return websWrite(wp, T("%d"), 1);
			break;
		case KPB_Middle:
			return websWrite(wp, T("%d"), 2);
			break;
		case KPB_High:
			return websWrite(wp, T("%d"), 3);
			break;
		default :
			return websWrite(wp, T(""));
			break;
	}
	return websWrite(wp, T(""));
}


int GetMaxproModeAddr()
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYMODEADDR;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    char *p = recvbuf;
    int modeaddr = atoi(p);
    return modeaddr;

	//return UltraKeyPrivData.maxpro.KeyboardAddr;
}

int GetMaxproPort(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYPORT;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    char *p = recvbuf;
    int port = atoi(p);
    return port;
}

int GetMaxproJoyStickSpeed(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYJOYSTICKSPEED;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    //printf("GetMaxproJoyStickSpeed=%d\n",recvbuf[0]);
    return recvbuf[0];
}

int GetMaxproSerialPortBaudrate(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYSERIALPORTBAUDRATE;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    return recvbuf[0];
}

int GetMaxproSerialPortMode(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYSERIALPORTMODE;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    return recvbuf[0];
}

PtrEtherNetwork_t GetMaxproIPaddr(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYIPADDR;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    return (PtrEtherNetwork_t)recvbuf;
}

int get_vb_ID(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYVBID;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    char *p = recvbuf;
    int id = atoi(p);
    return id;

}

int GetVbPort(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);

    sendbuf[0] = DISPLAYVBPORT;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    char *p = recvbuf;
    int port = atoi(p);
    //printf("DISPLAYVBPORT=%d\n",port);
    return port;

}

int get_vb_baudrate(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYBAUDRATE;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    //printf("DISPLAYBAUDRATE=%d\n",recvbuf[0]);
    return recvbuf[0];

}



PtrEtherNetwork_t GetVbIPaddr(void)
{
    int i ;
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYVBIPADDR;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    return (PtrEtherNetwork_t)recvbuf;
}


PtrEtherNetwork_t GetSysNetworkConfig(void)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYSYSNETWORKCONFIG;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
        recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    //printf("client receive=%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",\
    //recvbuf[0],recvbuf[1],recvbuf[2],recvbuf[3],recvbuf[4],recvbuf[5],recvbuf[6],recvbuf[7],recvbuf[8],\
   // recvbuf[9],recvbuf[10],recvbuf[11],recvbuf[12],recvbuf[13],recvbuf[14],recvbuf[15]);

    return (PtrEtherNetwork_t)recvbuf;

}

int SetMaxproModeAddr(int addr)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);

    sprintf(sendbuf ,"%c:%d", SETMAXPROMODEADDR, addr);

    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));

    return 0;
}


int SetMaxproIPAddr(PtrEtherNetwork_t addr)
{

    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETMAXPROIPADDR;
    sendbuf[1] = addr->IP[0];
    sendbuf[2] = addr->IP[1];
    sendbuf[3] = addr->IP[2];
    sendbuf[4] = addr->IP[3];
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetMaxproPort(int port)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf ,"%c:%d", SETMAXPROPORT, port);
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetMaxproJoyStickSpeed(unsigned char speed)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETMAXPROJOYSTICKSPEED;
    sendbuf[1] = speed;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetMaxproSerailPortBaudrate(unsigned char baudrate)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETMAXPROSERIALPORTBAUDRATE;
    sendbuf[1] = baudrate;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetMaxproSerialPortMode(unsigned char mode)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETMAXPROSERIALPORTMODE;
    sendbuf[1] = mode;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int set_vb_ID(int addr)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf ,"%c:%d", SETVBID, addr);
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int set_vb_baudrate(unsigned char baud)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETVBBAUDRATE;
    sendbuf[1] = baud;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetVbIPAddr(PtrEtherNetwork_t addr)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETVBIPADDR;
    sendbuf[1] = addr->IP[0];
    sendbuf[2] = addr->IP[1];
    sendbuf[3] = addr->IP[2];
    sendbuf[4] = addr->IP[3];
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetVbPort(int port)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf ,"%c:%d", SETVBPORT, port);
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


static int CheckIP(signed char * mark,int validnum)
{
	int i;
	int intValue = 0;
	for(i = 0; i < strlen((const char*)mark); i++)
	{
		if(mark[i] == '\n' || mark[i] == '\r')
		{
			mark[i] = 0;
		}
		else
		{
			if(mark[i] > '9'||mark[i] < '0')
				return -1;
		}
	}
	if(strlen((const char*)mark) > 3 || strlen((const char*)mark) <= 0)
		return -1;
	for(i = 0; i < strlen((const char*)mark); i++)
	{
		intValue = intValue * 10 + (mark[i] - '0');
		if(intValue > validnum) return -1;
	}

	return intValue;
}



static int getip(webs_t wp,signed char* var, int iptype[],int validnum)
{
	int i;
	signed char* sp;
	signed char varName[100];

	for(i = 0; i < 4; i++)
	{
		sprintf((char*)varName, "%s%d",(char*)var, i + 1);
		sp = (signed char*)websGetVar(wp, (char*)varName, T(""));
		if(*sp == 0){
			return -1;
		}
		if(i == 0)
			iptype[i] = CheckIP(sp,validnum);
		else
			iptype[i] = CheckIP(sp,255);
		if(iptype[i] < 0){
			return -1;
		}
	}

	return 1;
}

/*================================================================================

FUNCTION: set_ip          

DESCRIPTION: 
   Get IPv4 address.

ARGUMENTS PASSED:
   None.
  
RETURN VALUE:
   1 successful 0 fail
================================================================================*/
 

int set_ip(signed char * name, signed char * ip_addr)
{
	struct sockaddr_in *addr;
        struct ifreq ifr;
        int sockfd;
        
        if(ip_addr == NULL)
        	return 0;
 
        if((sockfd = socket(AF_INET,SOCK_DGRAM,0))<0)
        {
        //	DBG_PERROR("socket");
        	return 0;
        }
        strncpy(ifr.ifr_name, (char*)name,IFNAMSIZ-1);
        addr = ((struct sockaddr_in *)&(ifr.ifr_addr));
        bzero(addr, sizeof(struct sockaddr_in)); 
        if(inet_aton((const char*)ip_addr, &(addr->sin_addr))<0)
        	return 0;
     	addr->sin_family = AF_INET;
        if(ioctl(sockfd,SIOCSIFADDR,&ifr) == -1)
        {
          //  DBG_PERROR("ioctl");
	    close(sockfd);
            return 0;
        }

	 close(sockfd);
        return 1;
}


int SetSysIPAddr(PtrEtherNetwork_t ethernet)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETSYSIPADDR;
    sendbuf[1] = ethernet->IP[0];
    sendbuf[2] = ethernet->IP[1];
    sendbuf[3] = ethernet->IP[2];
    sendbuf[4] = ethernet->IP[3];
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}


int SetSysNetMask(PtrEtherNetwork_t ethernet)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETSYSNETMASK;
    sendbuf[1] = ethernet->NetMsk[0];
    sendbuf[2] = ethernet->NetMsk[1];
    sendbuf[3] = ethernet->NetMsk[2];
    sendbuf[4] = ethernet->NetMsk[3];
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}

int SetSysGateWay(PtrEtherNetwork_t ethernet)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETSYSGATEWAY;
    sendbuf[1] = ethernet->GateWay[0];
    sendbuf[2] = ethernet->GateWay[1];
    sendbuf[3] = ethernet->GateWay[2];
    sendbuf[4] = ethernet->GateWay[3];
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}

int SetSysDNS(PtrEtherNetwork_t ethernet)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = SETSYSDNS;
    sendbuf[1] = ethernet->DNS[0];
    sendbuf[2] = ethernet->DNS[1];
    sendbuf[3] = ethernet->DNS[2];
    sendbuf[4] = ethernet->DNS[3];
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    return 0;
}

////////////////functions////////////////////////
static int  aspShowVersion(int eid, webs_t wp, int argc, char_t **argv)
{
    a_assert(websValid(wp));
    signed char code_version[VERSION_STR_SIZ];
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sendbuf[0] = DISPLAYVER;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
    recvfrom(PrimaryUDP, (char *)code_version, sizeof(code_version), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    return websWrite(wp, T("%s"), code_version);
}


static int  aspDisplaySystemConfig(int eid, webs_t wp, int argc, char_t **argv)
{
	a_assert(websValid(wp));
    
	if (argc < 0)
	{
        return websWrite(wp, T(""));
	}
	if (strncmp("sysmode", argv[0], sizeof("sysmode")) == 0)
	{
		return aspDisplaySystemConfigSysMode(wp);
	}
	else if (strncmp("syslanguage", argv[0], sizeof("syslanguage")) == 0)
	{
		return websWrite(wp, T("%d"), 0);
	}
	else if (strncmp("background", argv[0], sizeof("background")) == 0)
	{
		return aspDisplaySystemConfigBackLight(wp);
	}
	else
	{
		return websWrite(wp, T(""));
	}

	return websWrite(wp, T(""));
}


static int  aspDisplayMaxpro(int eid, webs_t wp, int argc, char_t **argv)
{
	a_assert(websValid(wp));
	PtrEtherNetwork_t etherNet = NULL;

	if (argc < 0)
	{
		return websWrite(wp, T("%d"), 0);
	}

	if (strncmp("keyboardID", argv[0], sizeof("keyboardID")) == 0)
	{
		return websWrite(wp, T("%d"), GetMaxproModeAddr());
	}
	else if (strncmp("portnumber", argv[0], sizeof("portnumber")) == 0)
	{
		return websWrite(wp, T("%d"), GetMaxproPort());
	}
	else if (strncmp("joystick", argv[0], sizeof("joystick")) == 0)
	{
		return websWrite(wp, T("%d"), GetMaxproJoyStickSpeed());
	}
	else if (strncmp("joystick", argv[0], sizeof("joystick")) == 0)
	{
		return websWrite(wp, T("%d"), GetMaxproJoyStickSpeed());
	}
	else if (strncmp("baudrate", argv[0], sizeof("baudrate")) == 0)
	{
		return websWrite(wp, T("%d"), GetMaxproSerialPortBaudrate());
	}
	else if (strncmp("connectmode", argv[0], sizeof("connectmode")) == 0)
	{
		return websWrite(wp, T("%d"), GetMaxproSerialPortMode());
	}       
        else if (strncmp("netstatus", argv[0], sizeof("netstatus")) == 0)
        {
            char ip[64];
            etherNet = GetMaxproIPaddr();
            sprintf(ip,"ping -q -c 1 -s 1 %d.%d.%d.%d",etherNet->IP[0],etherNet->IP[1],etherNet->IP[2],etherNet->IP[3]);
            if (system(ip) != 0)
                return websWrite(wp, T("%d"), 0);
            else
                return websWrite(wp, T("%d"), 1);
        }
	else if (strstr(argv[0], "ipaddr") != NULL)
	{
		etherNet = GetMaxproIPaddr();
		if (etherNet == NULL)
		{
			return websWrite(wp, T("%d"), "0");
		}
		if (strncmp("ipaddr1", argv[0], sizeof("ipaddr1")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[0]);
		}
		if (strncmp("ipaddr2", argv[0], sizeof("ipaddr2")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[1]);
		}
		if (strncmp("ipaddr3", argv[0], sizeof("ipaddr3")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[2]);
		}
		if (strncmp("ipaddr4", argv[0], sizeof("ipaddr4")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[3]);
		}
		return websWrite(wp, T("%d"), "0");
	}
	else
	{
		return websWrite(wp, T("%d"), 0);
	}

	return websWrite(wp, T("%d"), 0);
}


static int  aspDisplayVideoBlox(int eid, webs_t wp, int argc, char_t **argv)
{
	a_assert(websValid(wp));
	PtrEtherNetwork_t etherNet = NULL;

	if (argc < 0)
	{
		return websWrite(wp, T("%d"), 0);
	}

	if (strncmp("keyboardID", argv[0], sizeof("keyboardID")) == 0)
	{
		return websWrite(wp, T("%d"), get_vb_ID());
	}
	else if (strncmp("portnumber", argv[0], sizeof("portnumber")) == 0)
	{
		return websWrite(wp, T("%d"), GetVbPort());
	}//qh Add Serial for VB
	else if (strncmp("baudrate", argv[0], sizeof("baudrate")) == 0)
	{
		switch(get_vb_baudrate()) {
			case SB1200:
				return websWrite(wp, T("%d"), 1);
				break;
			case SB9600:
				return websWrite(wp, T("%d"), 2);
				break;
			case SB19200:
				return websWrite(wp, T("%d"), 3);
				break;
			case SB57600:
				return websWrite(wp, T("%d"), 4);
				break;
			default:
				return websWrite(wp, T("%d"), 3);
				break;
		}	
	}
      else if (strncmp("netstatus", argv[0], sizeof("netstatus")) == 0)
      {
            char ip[64];
            etherNet = GetVbIPaddr();
            sprintf(ip,"ping -q -c 1 -s 1 %d.%d.%d.%d",etherNet->IP[0],etherNet->IP[1],etherNet->IP[2],etherNet->IP[3]);
            if (system(ip) != 0)
                return websWrite(wp, T("%d"), 0);
            else
                return websWrite(wp, T("%d"), 1);
      }
	else if (strstr(argv[0], "ipaddr") != NULL)
	{
		etherNet = GetVbIPaddr();
		if (etherNet == NULL)
		{
			return websWrite(wp, T("%d"), "0");
		}
		if (strncmp("ipaddr1", argv[0], sizeof("ipaddr1")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[0]);
		}
		if (strncmp("ipaddr2", argv[0], sizeof("ipaddr2")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[1]);
		}
		if (strncmp("ipaddr3", argv[0], sizeof("ipaddr3")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[2]);
		}
		if (strncmp("ipaddr4", argv[0], sizeof("ipaddr4")) == 0)
		{
			return websWrite(wp, T("%d"), etherNet->IP[3]);
		}
		return websWrite(wp, T("%d"), "0");
	}
	else
	{
		return websWrite(wp, T("%d"), 0);
	}

	return websWrite(wp, T("%d"), 0);
}


static int  aspDisplayIPAddr(int eid, webs_t wp, int argc, char_t **argv)
{
	int i = -1;
	a_assert(websValid(wp));
	PtrEtherNetwork_t ptrNet = NULL;
	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	i = atoi(argv[0]);
	ptrNet = GetSysNetworkConfig();
	if (ptrNet == NULL)
	{
		return websWrite(wp, T("%d"), "0");
	}

	return websWrite(wp, T("%d"), ptrNet->IP[i]);
}


static int  aspDisplayNetmask(int eid, webs_t wp, int argc, char_t **argv)
{
	int i = -1;
	a_assert(websValid(wp));
	PtrEtherNetwork_t ptrNet = NULL;

	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	i = atoi(argv[0]);
	ptrNet = GetSysNetworkConfig();
	if (ptrNet == NULL)
	{
		return websWrite(wp, T("%d"), "0");
	}

	return websWrite(wp, T("%d"), ptrNet->NetMsk[i]);
}


static int  aspDisplayGateWay(int eid, webs_t wp, int argc, char_t **argv)
{
	int i = -1;
	a_assert(websValid(wp));
	PtrEtherNetwork_t ptrNet = NULL;

	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	i = atoi(argv[0]);
	ptrNet = GetSysNetworkConfig();
	if (ptrNet == NULL)
	{
		return websWrite(wp, T("%d"), "0");
	}

	return websWrite(wp, T("%d"), ptrNet->GateWay[i]);
}


static int  aspDisplayDns(int eid, webs_t wp, int argc, char_t **argv)
{
	int i = -1;
	a_assert(websValid(wp));
	PtrEtherNetwork_t ptrNet = NULL;

	if (argc < 1)
	{
        	return websWrite(wp, T("%d"), "0");
	}

	i = atoi(argv[0]);
	ptrNet = GetSysNetworkConfig();
	if (ptrNet == NULL)
	{
		return websWrite(wp, T("%d"), "0");
	}

	return websWrite(wp, T("%d"), ptrNet->DNS[i]);
}


static int asp_DisplayHardKeyAction(int eid, webs_t wp, int argc, char_t **argv)
{
        struct timeval tv;
        fd_set readfds;
        tv.tv_sec = 1;
        tv.tv_usec = 0;
        FD_ZERO(&readfds);
        FD_SET(PrimaryUDP, &readfds);
        memset(sendbuf,0,sizeof(sendbuf));
        memset(recvbuf,0,sizeof(recvbuf));
        socklen_t fromlen = sizeof(ServerSock);

        sprintf(sendbuf,"%c:%d",DISPLAYHARDKEYACTION, atoi(argv[0]));
        sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
        recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	    
        return websWrite(wp, T("%s"), recvbuf);
}


static int asp_DisplayHardKeyActionIndex(int eid, webs_t wp, int argc, char_t **argv)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf,"%c:%d",DISPLAYHARDKEYACTIONINDEX, atoi(argv[0]));

    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    if(strlen(recvbuf))
    {
        return websWrite(wp, T("%d"), atoi(recvbuf));
    }
    else
    {
        return websWrite(wp, T("%s"),"");
    }

}

static int asp_DisplayHardKeyName(int eid, webs_t wp, int argc, char_t **argv)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);

    sendbuf[0] = DISPLAYHARDKEYNAME;

    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    return websWrite(wp, T("%d"),atoi(recvbuf));
      
}



static int asp_DisplayAction(int eid, webs_t wp, int argc, char_t **argv)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    //  signed char fun_string[MAX_FUNC_NAME_LEN*SYSTEMDEFAULT_FUN_NUM+256];
    char fun_string[32*1024+256];
    memset(fun_string,0,sizeof(fun_string));
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf,"%c:%d",DISPLAYACTION, atoi(argv[0]));
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)fun_string, sizeof(fun_string), 0, (struct sockaddr*)&ServerSock,&fromlen);	

    return websWrite(wp, T("%s"),fun_string);


}

static void form_HardkeyConfig(webs_t wp, char_t *path, char_t *query)
{
    char *action_index;
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);        
    action_index=websGetVar(wp,"Action_String_hiddenField",T(""));        
    sprintf(sendbuf,"%c:",FORMHARDKEYCONFIG);
    strcat(sendbuf,action_index);
    //printf("sendbuf=%s\n",sendbuf);
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    websRedirect(wp, T("/ConfigHardkeys.asp"));

}

static void form_ChangeHardKey(webs_t wp, char_t *path, char_t *query)
{
    hardKey_index=atoi(websGetVar(wp,"hardkey_name_hidden",T("")));
    printf("hardKey_index=%d\n",hardKey_index);
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    sprintf(sendbuf,"%c:%d",FORMCHANGEHARDKEY, hardKey_index);        
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
    websRedirect(wp, T("/ConfigHardkeys.asp"));
}

void RebootSystem()
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);

    sendbuf[0] = REBOOT;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));        
}



void *reboot_thread(void *arg)
{
    sleep(3);	
    RebootSystem();
    return 0;
}


static void formReboot(webs_t wp, char_t *path, char_t *query)
{
    pthread_t rebootId;
    websRedirect(wp, T("/reboot.htm"));
    if (pthread_create(&rebootId, 0, reboot_thread, NULL) != 0)
    {
    trace(3, "%s pthread_create reboot_thread error\n", __FUNCTION__);
    }
    pthread_detach(rebootId);
    return ;
}


static void formConfigMaxpro(webs_t wp, char_t *path, char_t *query)
{
	signed char *keyboardID = NULL;
	signed char ipaddr[256] = "";
	signed char *ipaddr1 = NULL;
	signed char *ipaddr2 = NULL;
	signed char *ipaddr3 = NULL;
	signed char *ipaddr4 = NULL;
	signed char *portnumber = NULL;
	signed char *joystick = NULL;
	signed char *baudrate = NULL;
	signed char *connectmode = NULL;
	int ip = 0;
	a_assert(websValid(wp));
	EtherNetwork_t etherNet;
	//printf("%s----%d %s\n", __FILE__, __LINE__, __FUNCTION__);

	memset(&etherNet, 0, sizeof(etherNet));
	keyboardID = (signed char*)websGetVar(wp,"keyboardID",T(""));
	ipaddr1 = (signed char*)websGetVar(wp,"ipaddr1",T(""));
	ipaddr2 = (signed char*)websGetVar(wp,"ipaddr2",T(""));
	ipaddr3 = (signed char*)websGetVar(wp,"ipaddr3",T(""));
	ipaddr4 = (signed char*)websGetVar(wp,"ipaddr4",T(""));
	joystick = (signed char*)websGetVar(wp,"joystick",T(""));
	portnumber = (signed char*)websGetVar(wp,"portnumber",T(""));
	baudrate = (signed char*)websGetVar(wp,"baudrate",T(""));
	connectmode = (signed char*)websGetVar(wp,"connectmode",T(""));
	sprintf((char*)ipaddr, "%s.%s.%s.%s", ipaddr1, ipaddr2, ipaddr3, ipaddr4);
	ip = inet_addr((const char*)ipaddr);
    
        Switch_to_frame();

	if (SetMaxproModeAddr(atoi((const char*)keyboardID)) != 0)
	{
		//trace(3, "%s SetMaxproModeAddr  error\n", __FUNCTION__);
	}

	memcpy(etherNet.IP, &ip, 4);
	if (SetMaxproIPAddr(&etherNet) != 0)
	{
	//	trace(3, "%s SetMaxproIPAddr  error\n", __FUNCTION__);
	}
	if (SetMaxproPort(atoi((const char*)portnumber)) != 0)
	{
	//	trace(3, "%s SetMaxproPort  error\n", __FUNCTION__);
	}
	if (SetMaxproJoyStickSpeed(atoi((const char*)joystick)) != 0)
	{
	//	trace(3, "%s SetMaxproJoyStickSpeed  error\n", __FUNCTION__);
	}
	if (SetMaxproSerailPortBaudrate(atoi((const char*)baudrate)) != 0)
	{
		//trace(3, "%s SetMaxproSerailPortBaudrate  error\n", __FUNCTION__);
	}
	if (SetMaxproSerialPortMode(atoi((const char*)connectmode)) != 0)
	{
		//trace(3, "%s SetMaxproSerialPortMode  error\n", __FUNCTION__);
	}
	websRedirect(wp, T("/Maxpro.asp"));
	return ;
}



static void formConfigVideoBlox(webs_t wp, char_t *path, char_t *query)
{
    signed char *keyboardID = NULL;
    signed char ipaddr[256] = "";
    signed char *ipaddr1 = NULL;
    signed char *ipaddr2 = NULL;
    signed char *ipaddr3 = NULL;
    signed char *ipaddr4 = NULL;
    //qh Add Serial for VB
    signed char *baudrate = NULL;
    SelectableSerialBaudRate_t baudRate;
    //
    signed char *portnumber = NULL;
    int ip = 0;
    a_assert(websValid(wp));
    EtherNetwork_t etherNet;
    //printf("%s----%d %s\n", __FILE__, __LINE__, __FUNCTION__);

    memset(&etherNet, 0, sizeof(etherNet));
    keyboardID = (signed char*)websGetVar(wp,"keyboardID",T(""));
    ipaddr1 = (signed char*)websGetVar(wp,"ipaddr1",T(""));
    ipaddr2 = (signed char*)websGetVar(wp,"ipaddr2",T(""));
    ipaddr3 = (signed char*)websGetVar(wp,"ipaddr3",T(""));
    ipaddr4 = (signed char*)websGetVar(wp,"ipaddr4",T(""));
    //qh Add Serial for VB
    baudrate = (signed char*)websGetVar(wp,"baudrate",T(""));
    portnumber = (signed char*)websGetVar(wp,"portnumber",T(""));
    sprintf((char*)ipaddr, "%s.%s.%s.%s",(char*)ipaddr1, (char*)ipaddr2,(char*)ipaddr3, (char*)ipaddr4);
    ip = inet_addr((const char*)ipaddr);
    Switch_to_frame();
    if (set_vb_ID(atoi((const char*)keyboardID)) != 0)
    {
    //	trace(3, "%s set_vb_ID  error\n", __FUNCTION__);
    }
    //qh Add Serial for VB	
    switch(atoi((const char*)baudrate)) 
    {
        case 1:
        baudRate = SB1200;
        break;
        case 2:
        baudRate = SB9600;
        break;
        case 3:
        baudRate = SB19200;
        break;
        case 4:
        baudRate = SB57600;
        break;
        default:
        baudRate = SB19200;
        break;
    }	

    if (set_vb_baudrate(baudRate) != 0)
    {
    //	trace(3, "%s set_vb_baudrate  error\n", __FUNCTION__);
    }
    //
    memcpy(etherNet.IP, &ip, 4);
    if (SetVbIPAddr (&etherNet) != 0)
    {
    //		trace(3, "%s SetVbIPAddr  error\n", __FUNCTION__);
    }
    if (SetVbPort(atoi((const char*)portnumber)) != 0)
    {
    //		trace(3, "%s SetVbPort  error\n", __FUNCTION__);
    }
    websRedirect(wp, T("/VideoBlox.asp"));
    return ;
}


static void formIPConfig(webs_t wp, char_t *path, char_t *query)
{
    int netaddress[4];
    int netmask[4];
    int gateway[4];
    int DNS[4];
    int i = -1;
    signed char chAddr[4];
    signed char chNetmask[4];
    signed char chGateway[4];
    signed char chDNS[4];
    EtherNetwork_t etherNetwork;
    memset(&etherNetwork, 0, sizeof(etherNetwork));
    memset(netaddress, 0, 4 * sizeof(int));
    memset(netmask, 0, 4 * sizeof(int));
    memset(gateway, 0, 4 * sizeof(int));
    memset(DNS, 0, 4 * sizeof(int));
    memset(chAddr, 0, 4 * sizeof(signed char));
    memset(chNetmask, 0, 4 * sizeof(signed char));
    memset(chGateway, 0, 4 * sizeof(signed char));
    memset(chDNS, 0, 4 * sizeof(signed char));
    if(getip(wp, (signed char*)"ipaddr", netaddress, 239) < 0)
    {
        websRedirect(wp, T("/ConfigDispIPError.asp"));
        websDone(wp, 200);
        return;
    }

    if(getip(wp, (signed char*)"submsk",netmask, 255) < 0)
    {
        websRedirect(wp, T("/ConfigDispIPError.asp"));
        websDone(wp, 200);
        return;
    }

    if(getip(wp, (signed char*)"getway", gateway, 239) < 0)
    {
        websRedirect(wp, T("/ConfigDispIPError.asp"));
        websDone(wp, 200);
        return;
    }
    if(getip(wp, (signed char*)"dns", DNS, 239) < 0)
    {
        websRedirect(wp, T("/ConfigDispIPError.asp"));
        websDone(wp, 200);
        return;
    }

    for (i = 0; i < 4; i ++)
    {
        chAddr[i] = netaddress[i];
        chNetmask[i] = netmask[i];
        chGateway[i] = gateway[i];
        chDNS[i] = DNS[i];
    }
    memcpy(etherNetwork.IP, chAddr, 4 * sizeof(signed char));
    memcpy(etherNetwork.NetMsk, chNetmask, 4 * sizeof(signed char));
    memcpy(etherNetwork.GateWay, chGateway, 4 * sizeof(signed char));
    memcpy(etherNetwork.DNS, chDNS, 4 * sizeof(signed char));
    Switch_to_frame();	
    if (SetSysIPAddr(&etherNetwork) != 0)
    {
    //trace(3, "%s SetSysIPAddr error\n", __FUNCTION__);
    }

    if (SetSysNetMask(&etherNetwork) != 0)
    {
    //trace(3, "%s SetSysNetMask error\n", __FUNCTION__);
    }

    if (SetSysGateWay(&etherNetwork) != 0)
    {
    //trace(3, "%s SetSysGateWay error\n", __FUNCTION__);
    }

    if (SetSysDNS(&etherNetwork) != 0)
    {
    //trace(3, "%s SetSysDNS error\n", __FUNCTION__);
    }
    websDone(wp, 200);
    return;
}


static void formConfigSystem(webs_t wp, char_t *path, char_t *query)
{
    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);
    char *systemMode = NULL;
    char *systemLanguage = NULL;
    char *systemBackground = NULL;
    systemMode = websGetVar(wp,"sysmode",T(""));
    systemLanguage = websGetVar(wp,"syslanguage",T(""));
    systemBackground = websGetVar(wp,"background",T(""));

    memset(sendbuf,0,sizeof(sendbuf));
    memset(recvbuf,0,sizeof(recvbuf));
    socklen_t fromlen = sizeof(ServerSock);
    
    sendbuf[0] = CONFIGSYSTEM;
    sendbuf[1] = *systemMode;
    sendbuf[2] = *systemLanguage;
    sendbuf[3] = *systemBackground;
    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
    recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);
    websRedirect(wp, T("/ConfigSystem.asp"));
    return ;

}

static int printError(webs_t wp,int errorNumber)
{
	switch (errorNumber)
	{
		case 1:
			websWrite(wp,T(""));
			break;
		case 2:
			websWrite(wp,T("Error: Fail to change password."));
			break;
		case 3:
			websWrite(wp,T("Error: Incorrect old password."));
			break;
		case 4:
			websWrite(wp,T("Error: New password should not be empty."));
			break;
		case 5:
			websWrite(wp,T("Error: New password is too long."));
			break;
		case 6:
			websWrite(wp,T("Error: New passwords are not the same."));
			break;
		case 7:
			websWrite(wp,T("Error: User password saving error."));
			break;
		case 8:
			websWrite(wp,T("Change password success."));
			break;
		case 9:
			websWrite(wp,T("Error: Can't change password."));
			break;
		case 10:
			websWrite(wp,T("Error: Can't delete user."));
			break;
		case 11:
			websWrite(wp,T("Delete  user success."));
			break;
		case 12:
			websWrite(wp,T("Error: Have the same user."));
			break;
		case 13:
			websWrite(wp,T("Error: Can't add user."));
			break;
		case 14:
			websWrite(wp,T("Add user success."));
			break;
		/* FIX: only A-Z a-z and 0-9 can be used as password. 20110303 */
		case 15:
			websWrite(wp,T("Error: only A-Z, a-z and 0-9 can be used as password."));
			break;
		default:
			websWrite(wp,T(""));
			break;
	}
	return 0;
}
 
static int  aspCheckChangePass(int eid, webs_t wp, int argc, char_t **argv)
{
    printError(wp, changePassworkError);
    changePassworkError = 0;
    return 0;
}

static void formChangePassword(webs_t wp, char_t *path, char_t *query)
{
    char_t* oldpassword;
    char_t* newpassword;
    char_t* newpassword1;

    oldpassword = websGetVar(wp,"OldPass",T(""));
    newpassword = websGetVar(wp,"NewPass",T(""));
    newpassword1 = websGetVar(wp,"ConfirmPass",T(""));

    if(*oldpassword == 0 && *newpassword == 0 && *newpassword1 == 0)
    {
    changePassworkError = 1;
    return websRedirect(wp, T("/ChangePass.asp"));
    }

    if(*oldpassword == 0 || strlen(oldpassword) > 30)
    {
    changePassworkError = 3;//Incorrect Old Password
    return websRedirect(wp, T("/ChangePass.asp"));
    }

    if(*newpassword == 0)
    {
    changePassworkError = 4;//new password empty
    return websRedirect(wp, T("/ChangePass.asp"));
    }

    if(strlen(newpassword) > 30)
    {
    changePassworkError = 5;//new password too long
    return websRedirect(wp, T("/ChangePass.asp"));
    }

    /* FIX: only A-Z a-z and 0-9 can be used as password. 20110303 */
    char ii=0;
    for(;*(newpassword+ii)!='\0';ii++)
    {
    if(!((*(newpassword+ii)>='A' && *(newpassword+ii)<='Z') ||
    	(*(newpassword+ii)>='a' && *(newpassword+ii)<='z') ||
    	(*(newpassword+ii)>='0' && *(newpassword+ii)<='9')))
    {
    changePassworkError = 15;
    return websRedirect(wp, T("/ChangePass.asp"));
    }
    }

    if (umUserExists("Administrator"))
    {
    char_t * userpass = umGetUserPassword("Administrator");

    if (userpass)
    {
    if (gstrcmp(oldpassword, userpass) != 0)
    {
    	changePassworkError = 3;//Incorrect Old Password
    	return websRedirect(wp, T("/ChangePass.asp"));
    }
    bfree (B_L, userpass);
    }
    }

    if(*newpassword1 == 0)
    {
    changePassworkError = 6;//Passwords are not the same
    return websRedirect(wp, T("/ChangePass.asp"));
    }

    if (gstrcmp(newpassword, newpassword1) != 0)
    {
    changePassworkError = 6;//Passwords are not the same
    return websRedirect(wp, T("/ChangePass.asp"));
    }

    umDeleteUser("Administrator");
    umAddUser("Administrator",newpassword, "administrator", 0, FALSE);

    if(umCommit(NULL) != 0)
    {
    changePassworkError = 7;//User Password save error.
    }
    else
    {
    changePassworkError = 8;//Password change success.
    }

    return websRedirect(wp, T("/ChangePass.asp"));
}

int aspCheckUser(int eid, webs_t wp, int argc, char_t **argv) 
{
    char_t* username;
    char_t* password;

    username = websGetVar(wp,"UserName",T(""));
    password = websGetVar(wp,"Authentication",T(""));
    if (!umUserExists(username)) {
    	trace(3, T("SEC: Unknown user <%s>\n"),	username);
    	websWrite(wp,T(""));
    	return 0;
    }
    if (password && * password) {
    	char_t * userpass = umGetUserPassword(username);
    	if (userpass) {
    		if (gstrcmp(password, userpass) != 0) {
    			return websWrite(wp,T(""));
    		}
    		bfree (B_L, userpass);
    	}
    	else {
    		return websWrite(wp,T(""));
    	}
    }
    return websWrite(wp, T("admin"));
}

static int check_filename(char* str, char* folder)
{
	char* p;
	char* file_ext = NULL;
	char* file_type = NULL;
	int calc_time = 0;
	int dot_calc = 0;
	int str_length;
	str_length = strlen(str);
	p = str + str_length;

	for(; p != str - 1; p--)
	{
		if((int) *p == 13 || (int) *p == 10 || *p == '\"')
			*p = 0;

		if(*p == '.')
		{
			dot_calc ++;
			if(dot_calc == 1)
			{
				*p = 0;
				file_ext = p + 1;
			}
		}
		if(*p == '_')
		{
			*p = 0;
			calc_time ++;
			if(calc_time == 1)
			{
				file_type = p+1;
			}
		}
		if(*p == '/' || *p == '\\')
		{
			break;
		}
	}
	if(calc_time != 1)
	{
		return - 1;
	}
	if(strstr(folder,"Application")!= NULL)
	{
		if((strcmp(file_ext,"jffs2") != 0) ||((strcmp(file_type,"App") != 0) && (strcmp(file_type,"app") != 0) && (strcmp(file_type,"APP") != 0)))
		{
			return -1;
		}
		return FILE_APPLICATION;
	}
	if(strstr(folder,"FPGA")!= NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||((strcmp(file_type,"FPGA") != 0) && (strcmp(file_type,"fpga") != 0) && (strcmp(file_type,"Fpga") != 0)))
		{

			return -1;
		}
		return FILE_FPGA;
	}
	if(strstr(folder,"Kernel")!=NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||( (strcmp(file_type,"KERNEL") != 0) && (strcmp(file_type,"kernel") != 0) && (strcmp(file_type,"Kernel") != 0)))
		{
			return -1;
		}
		return FILE_KERNEL;
	}
	if(strstr(folder,"FileSystem")!= NULL)
	{
		if((strcmp(file_ext,"jffs2") != 0) ||((strcmp(file_type,"FS") != 0) && (strcmp(file_type,"Fs") != 0) && (strcmp(file_type,"fs") != 0)))
		{
			return -1;
		}
		return FILE_FS;
	}

	if(strstr(folder,"BL")!= NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||((strcmp(file_type,"Loader") != 0) && (strcmp(file_type,"loader") != 0) && (strcmp(file_type,"LOADER") != 0)))
		{
			return -1;
		}
		return FILE_BL;
	}
	if(strstr(folder,"SliderFW")!=NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||( (strcmp(file_type,"SLIDERFW") != 0) && (strcmp(file_type,"sliderFW") != 0) && (strcmp(file_type,"SliderFW") != 0)))
		{
			return -1;
		}

		return FILE_SILDERFW;
	}

	if(strstr(folder,"UltraKeyPlusFW")!=NULL)
	{
		return FILE_FIRMWARE_ULTRAKEYPLUS;
	}
	if(strstr(folder,"ClientFW")!=NULL)
	{
		return FILE_FIRMWARE_CLIENT;
	}
	if(strstr(folder,"configuration")!=NULL)
	{
		return FILE_DYNAMIC_BUTTON;
	}
	return -1;
}

static int mtd_burn_image(int file_length_write, char *mtd_device, FILE* fp)
{
	int device_fd = -1;
	unsigned char  *tmp_buf = NULL;
	long buf_size = 4 * 1024;
	long image_size = 0;
	mtd_info_t meminfo;
	erase_info_t erase;

	memset(&meminfo, 0, sizeof(meminfo));
	memset(&erase, 0, sizeof(erase));
	tmp_buf = (unsigned char *)malloc(buf_size);
	if(tmp_buf == NULL)
	{
		return -1;
	}
	memset(tmp_buf, 0, buf_size);
	device_fd = open(mtd_device, O_RDWR | O_SYNC);
	if(device_fd < 0)
	{
		free(tmp_buf);
		return -1;
	}

	//get the information of the image
	image_size = file_length_write;
	// Erase the mtd device first
	if(ioctl(device_fd, MEMGETINFO, &meminfo) != 0)
	{
		return -1;
	}
	erase.length = meminfo.erasesize;
	file_length = meminfo.size;
	//printf("meminfo.erasesize=%x.................meminfo.size=%x\n",meminfo.erasesize,meminfo.size);

	upload_type  = UPLOAD_PROCESS_TYPE_ERASE;
	for (erase.start = 0; erase.start < meminfo.size;erase.start += meminfo.erasesize)
	{
		upload_length = erase.start;
		if(ioctl(device_fd, MEMERASE, &erase) != 0)
		{
			return -1;
		}
	}

	//Write MTD device
	upload_type = UPLOAD_PROCESS_TYPE_WRITE;

	file_length = image_size;
	upload_length = file_length - image_size;

	while(image_size > 0)
	{
		int tmp,ret;

		tmp=fread(tmp_buf, 1, buf_size, fp);
		if(tmp <=0)
		{
			free(tmp_buf);
			printf("%s, %d, tmp=%x pos=%x image_size=%x\n", __FILE__, __LINE__,tmp,ftell(fp),image_size);
			fclose(fp);
			close(device_fd);
			return -1;
		}
		ret=write(device_fd, tmp_buf, tmp);
		if (ret!=tmp)
		{
			free(tmp_buf);
			printf("%s, %d, tmp=%x ret=%x pos=%x image_size=%x\n", __FILE__, __LINE__,tmp,ret,ftell(fp),image_size);
			fclose(fp);
			close(device_fd);
			return -1;
		}
		image_size -= tmp;

		upload_length = file_length - image_size;
	}
	free(tmp_buf);
	close(device_fd);
	return 0;
}

void *write_flash(void *arg)
{
	FILE *       fp;
	char buf[512] = " ";
	char folder[50] = " ";
	char * point = NULL;
	int delNum = 0;
	int fileLength = -1;

	total_length = globalPostLen;

	fp = fopen(tmpFile,"r");
	if(fp == NULL)
	{
		//file open error
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);
		return 0;
	}
	// first line is MIME boundary
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum = strlen(buf) * 3 + 4;
	// second line contains "Content-Disposition
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(buf);
	// third line is the enter
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(buf);
	//fourth line is the folder name
	if (fgets(folder, sizeof(folder), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(folder);
	if ((point=strchr(folder, '\r')) != NULL)
	{
		*point = '\0';
	}
	if ((point=strchr(folder, '\n')) != NULL)
	{
		*point = '\0';
	}

	//fifth line is the boundary
	if (fgets(&buf[2], sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	// sixth line is Content-Disposition and contain file name:
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(buf);
	point = strstr(buf, "filename=");
	if (point == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	//check file name and folder name
	point += strlen("filename=");
	if((folderType = check_filename(point,folder)) == -1)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	while (fgets(buf, sizeof(buf), fp) != NULL)
	{
		delNum += strlen(buf);
		if (strlen(buf) <= 2)
			break;
	}
	if(total_length < 0)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}
	fileLength = total_length - delNum;

	if(fileLength <= 0)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	if(folderType == FILE_APPLICATION)
	{
		if(mtd_burn_image(fileLength,APPLICATION_DEVICE,fp) < 0)
		{
			uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
		}
		else
		{
			uploadStatus = UPLOAD_STATUS_OK;
		}
	}
	else if(folderType == FILE_FPGA)
	{
		if(mtd_burn_image(fileLength,FPGA_DEVICE,fp) < 0)
		{
			uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
		}
		else
		{
			uploadStatus = UPLOAD_STATUS_OK;
		}
	}
	else if(folderType == FILE_KERNEL)
	{
		if(mtd_burn_image(fileLength,KERNEL_DEVICE,fp) < 0)
		{
			uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
		}
		else
		{
			uploadStatus = UPLOAD_STATUS_OK;
		}
	}
    else if(folderType == FILE_FS)
    {
    	if(mtd_burn_image(fileLength,FS_DEVICE,fp) < 0)
        {
        	uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
        }
        else
        {
            uploadStatus = UPLOAD_STATUS_OK;
        }
    }    
	else
	{
		//should not come here
		uploadStatus = UPLOAD_STATUS_ERROR;
	}
	//close and delete the tmp file.
	if(fp != NULL)
	{
		fclose(fp);
		fp = NULL;
	}
	unlink(tmpFile);
	return 0;
}

static void formUpload(webs_t wp, char_t *path, char_t *query)
{
    system("cp /sbin/reboot /tmp/reboot");
    a_assert(websValid(wp));
    FILE *fp = fopen(tmpFile, "wb+");
    if (fp == NULL)
        return;
    if (fwrite(globalPostData, 1, globalPostLen, fp) != globalPostLen)
        return;
    fclose(fp);
    uploadStatus = UPLOAD_STATUS_CONNTINUE;
    websRedirect(wp, T("/uploadstatus.asp"));
    Switch_to_frame();
    pthread_t thread;
    pthread_create(&thread, 0, write_flash, NULL);
    pthread_detach(thread);
    return;
    
}

void formUploadStatus(webs_t wp, char_t *path, char_t *query)
{
	websRedirect(wp, T("/uploadstatus.asp"));
	return ;
}

int  aspJump(int eid, webs_t wp, int argc, char_t **argv)
{
	/*
	 * uploadStatus = UPLOAD_STATUS_CONNTINUE :not wrong at present, need jump
	 * uploadStatus = UPLOAD_STATUS_OK :successful
	 */
	if(uploadStatus == UPLOAD_STATUS_CONNTINUE)
	{
		websWrite(wp,T("<meta http-equiv=\"refresh\" content=\"3; url=/uploadstatus.asp\">"));
	}
	else
	{
		websWrite(wp,T(""));
	}

	return 1;
}

int  aspShowProgress(int eid, webs_t wp, int argc, char_t **argv)
{
	/*
	 * uploadStatus = UPLOAD_STATUS_CONNTINUE :not wrong at present, need jump
	 * uploadStatus = UPLOAD_STATUS_OK :successful
	 */
	int percent = 0;
	int i = 0;

	SladerWait ++;
	if(uploadStatus == UPLOAD_STATUS_CONNTINUE)
	{
		if(file_length == 0)
		{
			percent = 0;
		}
		else
		{
			percent = (upload_length * 100) / file_length;
		}
		if(upload_type  == UPLOAD_PROCESS_TYPE_ERASE)
		{
			websWrite(wp,T("Erasing old files: %d %c"), percent,'%');
		}
		else if(upload_type  == UPLOAD_PROCESS_TYPE_WRITE)
		{
			websWrite(wp,T("Copying new files: %d %c"), percent,'%');
		}
		else if(upload_type  == UPLOAD_PROCESS_TYPE_WAIT)
		{
			websWrite(wp,T("Please wait "));
			for (i = 0; i <= SladerWait; i ++)
			{
				websWrite(wp,T("."));
			}
		}
	}
	else
	{
		if(uploadStatus == UPLOAD_STATUS_ERROR)
		{
			uploadStatus = UPLOAD_STATUS_END;
			websWrite(wp, T("Error: Upgrade failed."));
		}
		if(uploadStatus == UPLOAD_STATUS_OK)
		{
			uploadStatus = UPLOAD_STATUS_END;
			/* 
			 * FIX: use different messages for kernel/fs/app upgrading 
			 * per Sean's beta feedback. 
			 * 20110407
			 */
			//websWrite(wp,T("Upgrade success: Please Power Off UltraKeyPlus and then Power On!"));
			if(FILE_APPLICATION==folderType)
			{
				websWrite(wp,T("Upgrade success: System will reboot automatically, Please wait..."));
			}
			else if(FILE_KERNEL==folderType)
			{
				websWrite(wp,T("Upgrade success: System will reboot automatically, Please wait..."));
			}
			else if(FILE_FS==folderType)
			{
				websWrite(wp,T("Upgrade success: UK+ is rebooting, please reload browser once rebooted!"));
			}
			//wp->rebootFlag = REBOOTFLAG;
			system("/tmp/reboot");
		}
		websWrite(wp,T(""));
	}

	return 1;
}



///////////////////////over///////////////////////////
class SymHashEntry : public MprHashEntry {
  public:
	sym_t			sym;
					SymHashEntry(char *key, value_t *vp);
	virtual			~SymHashEntry();
	value_t			*getValue() { return &sym.content; };
};


class WebsForm : public MaEgiForm {
  private:
	WebsFormCb		goFormCallback;

  public:
					WebsForm(char *formName, WebsFormCb fn);
					~WebsForm();
	void			run(MaRequest *rq, char *script, char *path, 
						char *query, char *postData, int postLen);
};

//
//	Single threaded
//
static MprFile 		*file;

static MaServer		*defaultServer;
static MaHost		*defaultHost;
static WebsForm		*websForms[MPR_HTTP_MAX_GO_FORM];
static int 			maxForm;

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// CompatModule /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int mprCompatInit(void *handle)
{
	if (maGetHttp() == 0) {
		return MPR_ERR_NOT_INITIALIZED;
	}
	new MaCompatModule(handle);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

/********************************* ASP ********************************/
/*
 *	Typcial asp function. Usage "aspTest name address"
 */

static int aspTest(int eid, webs_t wp, int argc, char_t **argv)
{
	a_assert(websValid(wp));

    struct timeval tv;
    fd_set readfds;
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

	memset(sendbuf,0,sizeof(sendbuf));
	memset(recvbuf,0,sizeof(recvbuf));
	socklen_t fromlen = sizeof(ServerSock);

	if (strncmp("sysmode", argv[0], sizeof("sysmode")) == 0)
	{
    	sendbuf[0] = DISPLAYSYSMODE;

    	sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
		if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
			recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);	
		websWrite(wp, T("%d"), recvbuf[0]);
	}
	else if (strncmp("syslanguage", argv[0], sizeof("syslanguage")) == 0)
	{
        sendbuf[0] = DISPLAYSYSLAN;

        sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
        if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
            recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);
        websWrite(wp, T("%d"), recvbuf[0]);
	}
	else if (strncmp("background", argv[0], sizeof("background")) == 0)
	{
        sendbuf[0] = DISPLAYSYSBG;

        sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
        if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
            recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);
		websWrite(wp, T("%d"), recvbuf[0]);
	}
	else
	{
		return websWrite(wp, T(""));
	}
	return websWrite(wp, T(""));
}

/******************************* Goforms ******************************/
/*
 *	Typcial GoForm function. Parameters name address
 */

static void formTest(webs_t wp, char_t *path, char_t *query)
{
	char_t *systemMode = NULL;
	char_t *systemLanguage = NULL;
	char_t *systemBackground = NULL;

	systemMode = websGetVar(wp,"sysmode",T(""));
	systemLanguage = websGetVar(wp,"syslanguage",T(""));
	systemBackground = websGetVar(wp,"background",T(""));
	memset(sendbuf,0,sizeof(sendbuf));
	sendbuf[0] = CONFIGSYSTEM;
	strcat(sendbuf,":");
	strcat(sendbuf, systemMode);
	strcat(sendbuf, ":");
	strcat(sendbuf, systemLanguage);
    strcat(sendbuf, ":");
    strcat(sendbuf, systemBackground);

    sendto(PrimaryUDP, (const char*)&sendbuf, sizeof(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));
	websRedirect(wp, T("/AppwebTest.asp"));
}
////////////////////////////////////////////////////////////////////////////////

MaCompatModule::MaCompatModule(void *handle) : MaModule("gaCompat", handle)
{
	file = new MprFile();
	/* Asp Define */
        umOpen();
	umRestore(T("config/umconfig.txt"));
	websAspDefine(T("aspTest"), aspTest);
	websFormDefine(T("formTest"), formTest);
	/* ASP local socket set */
	PrimaryUDP = mksock(SOCK_DGRAM);
	BindSock(PrimaryUDP);
	bzero(&ServerSock,sizeof(struct sockaddr_in));
	ServerSock.sin_family = AF_INET;
	ServerSock.sin_addr.s_addr = inet_addr(SERVER_IP);
	ServerSock.sin_port = htons(SERVER_PORT);
   
        websAspDefine(T("DisplaySystemConfig"), aspDisplaySystemConfig);
        websAspDefine(T("DisplayMaxpro"), aspDisplayMaxpro);
        websAspDefine(T("DisplayVideoBlox"), aspDisplayVideoBlox);
        websAspDefine(T("DisplayIPAddr"), aspDisplayIPAddr);
        websAspDefine(T("DisplayNetmask"), aspDisplayNetmask);
	websAspDefine(T("DisplayGateWay"), aspDisplayGateWay);
	websAspDefine(T("DisplayDns"), aspDisplayDns);

    
	websFormDefine(T("ConfigSystem"), formConfigSystem);
        websFormDefine(T("ConfigMaxpro"), formConfigMaxpro);
	websFormDefine(T("ConfigVideoBlox"), formConfigVideoBlox);
    	websFormDefine(T("IPConfig"), formIPConfig);

        websAspDefine(T("ShowVersion"), aspShowVersion);
        websAspDefine(T("checkUser"), aspCheckUser);
        websAspDefine(T("checkChangePassword"), aspCheckChangePass);
        websFormDefine(T("ChangePassword"), formChangePassword);
        websFormDefine(T("UploadStatus"), formUploadStatus);
        websFormDefine(T("Reboot"), formReboot);
        websFormDefine(T("Upload"), formUpload);
        websAspDefine(T("Jump"), aspJump);
        websAspDefine(T("ShowProgress"), aspShowProgress);

        websAspDefine(T("DisplayHardKeyAction"), asp_DisplayHardKeyAction);
	websAspDefine(T("DisplayHardKeyActionIndex"), asp_DisplayHardKeyActionIndex);
	websAspDefine(T("DisplayHardKeyName"), asp_DisplayHardKeyName);
	websAspDefine(T("DisplayAction"), asp_DisplayAction);

    	websFormDefine(T("HardkeyConfig"), form_HardkeyConfig);
        websFormDefine(T("ChangeHardKey"), form_ChangeHardKey);
    
    
}

////////////////////////////////////////////////////////////////////////////////

MaCompatModule::~MaCompatModule()
{
	//
	//	No need to delete websForms[]. EgiHandler will delete these for us
	//
	delete file;
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// WebsForm ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

WebsForm::WebsForm(char *formName, WebsFormCb fn) : MaEgiForm(formName)
{
	goFormCallback = fn;
}

////////////////////////////////////////////////////////////////////////////////

WebsForm::~WebsForm()
{
}

////////////////////////////////////////////////////////////////////////////////

void WebsForm::run(MaRequest *rq, char *script, char *uri, 
	char *query, char *postData, int postLen)
{
	int		code;
    
	globalPostData = postData;
	globalPostLen = postLen;


	//
	//	GoAhead GoForms write their own headers
	//
	rq->setHeaderFlags(MPR_HTTP_HEADER_WRITTEN, 0);
#if BLD_FEATURE_KEEP_ALIVE
	rq->setNoKeepAlive();
#endif

	(*goFormCallback)((webs_t) rq, (char*) uri, (char*) query);

	//
	//	User must call websDone or websRedirect in the goform to complete
	//	the request. However, websRedirect does not close the request in
	//	Appweb. If websRedirect has not been called, then we set a flag
	//	to cause EGI to keep the request open.
	//
	code = rq->getResponseCode();
	if (300 < code && code <= 399) {
		//
		//	We also need to output headers if redirect is called.
		//

		rq->setHeaderFlags(0, MPR_HTTP_HEADER_WRITTEN);

	} else {
		//
		//	This will stop EGI from closing the socket
		//
		rq->setFlags(MPR_HTTP_DONT_FINISH, -1);
	}
}

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// C APIs ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
extern "C" {

char *strlower(char *string)
{
	char	*s;

	a_assert(string);

	if (string == NULL) {
		return NULL;
	}

	s = string;
	while (*s) {
		if (isupper(*s)) {
			*s = (char) tolower(*s);
		}
		s++;
	}
	*s = '\0';
	return string;
}

////////////////////////////////////////////////////////////////////////////////

char *strupper(char *string)
{
	char	*s;

	a_assert(string);
	if (string == NULL) {
		return NULL;
	}

	s = string;
	while (*s) {
		if (islower(*s)) {
			*s = (char) toupper(*s);
		}
		s++;
	}
	*s = '\0';
	return string;
}

////////////////////////////////////////////////////////////////////////////////

value_t valueInteger(long value)
{
	value_t	v;

	memset(&v, 0x0, sizeof(v));
	v.valid = 1;
	v.type = integer;
	v.value.integer = value;
	return v;
}

////////////////////////////////////////////////////////////////////////////////

value_t valueString(char* value, int flags)
{
	value_t	v;

	memset(&v, 0x0, sizeof(v));
	v.valid = 1;
	v.type = string;
	if (flags & VALUE_ALLOCATE) {
		v.allocated = 1;
		v.value.string = bstrdup(B_L, value);
	} else {
		v.allocated = 0;
		v.value.string = value;
	}
	return v;
}

////////////////////////////////////////////////////////////////////////////////

int emfSchedCallback(int delay, emfSchedProc *proc, void *arg)
{
	new MprTimer(delay, (MprTimerProc) proc, (void*) arg, 
		MPR_TIMER_AUTO_RESCHED);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void emfUnschedCallback(int id)
{
	MprTimer	*timer;

	timer = (MprTimer*) id;
	timer->stop(1000);
	timer->dispose();
}

////////////////////////////////////////////////////////////////////////////////

void emfReschedCallback(int id, int delay)
{
	MprTimer	*timer;

	timer = (MprTimer*) id;
	timer->reschedule(delay);
}

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// SymHashEntry /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

SymHashEntry::SymHashEntry(char *key, value_t *vp) : MprHashEntry(key)
{
	sym.forw = 0;
	sym.name.value.string = mprStrdup(key);
	sym.name.type = string;
	sym.arg = 0;

	sym.content = *vp;
	if (vp->allocated) {
		sym.content.value.string = mprStrdup(vp->value.string);
	}
}

////////////////////////////////////////////////////////////////////////////////

SymHashEntry::~SymHashEntry()
{
	if (sym.content.allocated) {
		mprFree(sym.content.value.string);
	}
	mprFree(sym.name.value.string);
}

////////////////////////////////////////////////////////////////////////////////

sym_fd_t symOpen(int tableSize)
{
	return (sym_fd_t) new MprHashTable();
}

////////////////////////////////////////////////////////////////////////////////

void symClose(sym_fd_t sd)
{
	MprHashTable	*table = (MprHashTable*) sd;

	delete table;
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symLookup(sym_fd_t sd, char *name)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->lookup(name);
	if (sp == 0) {
		return 0;
	}
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symEnter(sym_fd_t sd, char *name, value_t v, int arg)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = new SymHashEntry(name, &v);
	table->insert(sp);
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////

int symDelete(sym_fd_t sd, char *name)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->lookup(name);
	if (sp) {
		table->remove(sp);
		delete sp;
		return 0;
	} else {
		return -1;
	}
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symFirstEx(sym_fd_t sd, void **current)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->getFirst();
	if (sp == 0) {
		return 0;
	}
	* ((SymHashEntry**) current) = sp;
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symNextEx(sym_fd_t sd, void **current)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->getNext((SymHashEntry*) *current);
	if (sp == 0) {
		return 0;
	}
	* ((SymHashEntry**) current) = sp;
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////  User Management //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//	At the momemnt we are using the GoAhead WebServer UM module. This code
//	replaces that with a wrapper over the Appweb security model.
//
#if UNUSED

int umOpen() 
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void umClose() 
{
}

////////////////////////////////////////////////////////////////////////////////

int umRestore(char *filename)
{
	//	FUTURE -- can now implement
#if FUTURE
	auth->resetUserGroup();
	auth->readGroupFile(server, auth, filename);
	auth->readUserFile(server, auth, filename);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umCommit(char *filename)
{
#if FUTURE
	auth->save(filename);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umAddGroup(char *group, short privilege, accessMeth_t am, 
	bool_t protect, bool_t disabled)
{
#if FUTURE
	//
	//	protected == delete protected. This should be done in the UI anyway.
	//	accessMethod == noAuth, basic, digest
	//	disabled == enable the user
	//
	auth->createGroup(group);
	
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umAddUser(char *user, char *password, char *group, bool_t protect, 
	bool_t disabled)
{
#if FUTURE
	//	not supporting disabled, protect
	auth->addUserPassword(user, password);
	auth->addUsersToGroup(group, user);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umDeleteGroup(char *group)
{
#if FUTURE
	auth->removeGroup(group);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umDeleteUser(char *user)
{
#if FUTURE
	auth->removeUser(user);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetFirstGroup()
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetNextGroup(char *lastUser)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetFirstUser()
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetNextUser(char *lastUser)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

accessMeth_t umGetGroupAccessMethod(char *group)
{
	return AM_NONE;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umGetGroupEnabled(char *group)
{
	return 1;
}

////////////////////////////////////////////////////////////////////////////////

short umGetGroupPrivilege(char *group)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umGetUserEnabled(char *user)
{
	return 1;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetUserGroup(char *user)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetUserPassword(char *user)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umGroupExists(char *group)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetGroupAccessMethod(char *group, accessMeth_t am)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetGroupEnabled(char *group, bool_t enabled)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetGroupPrivilege(char *group, short privileges)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetUserEnabled(char *user, bool_t enabled)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetUserGroup(char *user, char *password)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetUserPassword(char *user, char *password)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umUserExists(char *user)
{
	return 0;
}

#endif // UNUSED 

////////////////////////////////////////////////////////////////////////////////

int websAspDefine(char *name, WebsAspCb fn)
{
	ejsDefineStringCFunction(-1, name, (MprStringCFunction) fn, 0, 
		MPR_VAR_ALT_HANDLE);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void websDecodeUrl(char *decoded, char *token, int len)
{
	maUrlDecode(decoded, len, token, 1, 1);
}

////////////////////////////////////////////////////////////////////////////////

void websDone(webs_t wp, int code)
{
	bool	closeSocket;

	closeSocket = 1;
	((MaRequest*) wp)->finishRequest(code, closeSocket);
}

////////////////////////////////////////////////////////////////////////////////

void websError(webs_t wp, int code, char *msg, ...)
{
	va_list		ap;
	char		buf[MPR_MAX_STRING];

	va_start(ap, msg);
	mprVsprintf(buf, sizeof(buf), msg, ap);
	((MaRequest*) wp)->requestError(code, buf);
	va_end(ap);
}

////////////////////////////////////////////////////////////////////////////////

char *websErrorMsg(int code)
{
	return (char*) maGetHttpErrorMsg(code);
}

////////////////////////////////////////////////////////////////////////////////

void websFooter(webs_t wp)
{
	((MaRequest*) wp)->write("</html>\n");
}

////////////////////////////////////////////////////////////////////////////////

int websFormDefine(char *name, WebsFormCb fn)
{
	char	nameBuf[MPR_HTTP_MAX_URL];

	if (maxForm >= MPR_HTTP_MAX_GO_FORM) {
		mprError(MPR_L, MPR_LOG, "Too many goForms");
		return -1;
	}
	mprSprintf(nameBuf, sizeof(nameBuf), "/goform/%s", name);
	websForms[maxForm++] = new WebsForm(nameBuf, fn);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *websGetDateString(websStatType *sbuf)
{
	if (sbuf) {
		MprFileInfo		info;
		info.mtime = sbuf->mtime;
		return (char*) maGetDateString(&info);
	} else {
		return (char*) maGetDateString(0);
	}
}

////////////////////////////////////////////////////////////////////////////////

char *websGetRequestLpath(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getFileName();
}

////////////////////////////////////////////////////////////////////////////////
//
//	Returns the decoded Uri path portion without query
//
char *websGetPath(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getUri();
}

////////////////////////////////////////////////////////////////////////////////

char *websGetUserName(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getUser();
}

////////////////////////////////////////////////////////////////////////////////
//
//	Returns a raw url un-decoded with query
//

char *websGetUrl(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getOriginalUri();
}

////////////////////////////////////////////////////////////////////////////////
#if 0
char *websGetVar(webs_t wp, char *var, char *def)
{
	MaRequest	*rq;
	char		*value;

	rq = (MaRequest*) wp;

	if ((value = rq->getVar(MA_FORM_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_HEADERS_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_REQUEST_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_SERVER_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_GLOBAL_OBJ, var, 0))  != 0) {
		return value;
	}
	return def;
}

#endif
////////////////////////////////////////////////////////////////////////////////

//void websSetVar(webs_t wp, char *var, char *value)
//{
//	MaRequest	*rq;

//	rq = (MaRequest*) wp;
//	rq->setVar(MA_GLOBAL_OBJ, var, value);
//}

////////////////////////////////////////////////////////////////////////////////

char_t *websGetRequestIpaddr(webs_t wp)
{
	MaRequest	*rq;

	rq = (MaRequest*) wp;
	return rq->getRemoteIpAddr();
}

////////////////////////////////////////////////////////////////////////////////

void websHeader(webs_t wp)
{
	MaRequest	*rq = (MaRequest*) wp;

	rq->write("HTTP/1.0 200 OK\r\n");
	rq->writeFmt("Server: %s\r\n", MPR_HTTP_SERVER_NAME);
	rq->write("Pragma: no-cache\r\n");
	rq->write("Cache-control: no-cache\r\n");
	rq->write("Content-Type: text/html\r\n");
	rq->write("\r\n");
	rq->write("<html>\r\n");
}

////////////////////////////////////////////////////////////////////////////////

int websPageOpen(webs_t wp, char *fileName, char *uri, int mode, int perm)
{
	return file->open(uri, mode, perm);
}

////////////////////////////////////////////////////////////////////////////////

int websPageClose(webs_t wp)
{
	file->close();
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int websPageStat(webs_t wp, char *fileName, char *uri, websStatType* sbuf)
{
	MaRequest		*rq;
	MprFileInfo		info;
 
	rq = (MaRequest*) wp;
	if (rq->host->server->fileSystem->stat(uri, &info) < 0) {
		return MPR_ERR_CANT_ACCESS;
	}
	sbuf->size = info.size;
	sbuf->isDir = info.isDir;
	sbuf->mtime = (time_t) info.mtime;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void websRedirect(webs_t wp, char *url)
{
	((MaRequest*) wp)->redirect(301, url);
	// ((MaRequest*) wp)->flushOutput(MPR_HTTP_FOREGROUND_FLUSH, 0);
}

////////////////////////////////////////////////////////////////////////////////

void websSetRealm(char *realmName)
{
	MaDir		*dir;
	MaAuth		*auth;

	dir = defaultHost->findBestDir(defaultServer->getServerRoot());

	if (dir == 0) {
		mprError(MPR_L, MPR_LOG,
			"websSetRealm Error: Server not yet configured");
		return;
	}

	auth = dir->getAuth();
	mprAssert(auth);
	if (auth == 0) {
		mprError(MPR_L, MPR_LOG,
			"webSetRealm Error: Server not yet configured");
		return;
	}

	auth->setRealm(realmName);
}

////////////////////////////////////////////////////////////////////////////////

void websSetRequestLpath(webs_t wp, char *fileName)
{
	((MaRequest*) wp)->setFileName(fileName);
}

////////////////////////////////////////////////////////////////////////////////

int websUrlHandlerDefine(char *urlPrefix, char *webDir, int arg, 
	int (*fn)(webs_t wp, char *urlPrefix, char *webDir, int arg, 
	char *url, char *path, char *query), int flags)
{
	mprError(MPR_L, MPR_LOG, "websUrlHandlerDefine is not supported");
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int websValid(webs_t wp)
{
	//
	//	Always return valid. Hope this is sufficient
	//
	return 1;
}

////////////////////////////////////////////////////////////////////////////////

int websValidateUrl(webs_t wp, char *path)
{
	if (maValidateUri(path) == 0) {
		return -1;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int websWrite(webs_t wp, char* fmt, ...)
{
	va_list		ap;
	char		buf[MPR_MAX_STRING];

	va_start(ap, fmt);
	mprVsprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);

	return ((MaRequest*) wp)->write(buf);
}

////////////////////////////////////////////////////////////////////////////////

int websWriteBlock(webs_t wp, char *buf, int nChars)
{
	return ((MaRequest*) wp)->write(buf, nChars);
}

////////////////////////////////////////////////////////////////////////////////
#if UNUSED_AND_INCOMPLETE

int websAspRequest(webs_t wp, char *localPath)
{
	MaRequest	*rq = (MaRequest*) wp;
	MprFileInfo	*info;
	char		*fileName, *errMsg;
	char		*docBuf;
	int			size;

	info = maGetRequestDocumentInfo(rq);

	size = info->size * sizeof(char);
	docBuf = (char*) mprMalloc(size + 1);
	docBuf[size] = '\0';

	if (maReadDoc(rq, docBuf, size) != size) {
		rq->requestError(404, "Can't read error document");
		mprFree(docBuf);
		return -1;
	}

	fileName = maGetRequestFileName();

	if (espProcessRequest(espRequest, fileName, docBuf, &errMsg) < 0) {
		if (espHandlerService->getFlags() & ESP_FLAGS_ERRORS_TO_BROWSER) {
			rq->writeFmt("<h2>ESP Error in \"%s\"</h2>\n", rq->getUriPath());
			rq->writeFmt("<p>In file: \"%s\"</p>\n", fileName);
			rq->writeFmt("<h3><pre>%s</pre></h3>\n", errMsg);
			rq->writeFmt("<p>To prevent errors being displayed in the "
				"browser, Put <b>\"EspErrors log\"</b> in the "
				"config file.</p>");
			mprFree(errMsg);
		} else {
			rq->requestError(404, 
				"Error processing ESP request %s\n: %s", fileName, 
				errMsg ? errMsg : "");
			if (errMsg) {
				mprFree(errMsg);
			}
			mprFree(docBuf);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}
	}
}

#endif // UNUSED
////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// Socket APIs ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void socketAcceptWrapper(void *data, MprSocket *sock, char *ip, 
	int port, MprSocket *listenSock, int isMprPoolThread)
{
	socketAccept_t	callback;

	mprAssert(sock);

	//
	//	We stored the user's accept callback function in the socket accept data.
	//
	sock->getAcceptCallback(0, (void**) &callback);

	(callback)((int) sock, ip, port, (int) listenSock);
}

////////////////////////////////////////////////////////////////////////////////

int socketOpenConnection(char *host, int port, socketAccept_t accept, int flags)
{
	MprSocket	*sock;
	int			rc, socketFlags;

	socketFlags = 0;
	if (flags & SOCKET_BROADCAST) {
		socketFlags |= MPR_SOCKET_BROADCAST;
	} else if (flags & SOCKET_DATAGRAM) {
		socketFlags |= MPR_SOCKET_DATAGRAM;
	} else if (flags & SOCKET_BLOCK) {
		socketFlags |= MPR_SOCKET_BLOCK;
	}

	sock = new MprSocket();
	mprAssert(sock);

	if (host == 0) {
		//
		//	We store the user's accept function in the socket accept data.
		//
		rc = sock->openServer(host, port, socketAcceptWrapper, (void*) accept, 
			socketFlags);
	} else {
		rc = sock->openClient(host, port, flags);
	}
	if (rc < 0) {
		return -1;
	}
	return (int) sock;
}

////////////////////////////////////////////////////////////////////////////////

void socketCloseConnection(int sid)
{
	MprSocket	*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	sock->close(MPR_SHUTDOWN_BOTH);
	sock->dispose();
}

////////////////////////////////////////////////////////////////////////////////

static void socketIoWrapper(void *data, MprSocket *sock, int socketMask, int
	isMprPoolThread)
{
	socketHandler_t		handler;
	int					mask, arg;

	mprAssert(sock);

	sock->getCallback(0, (void**) &handler, (void**) &arg, &mask);
	(handler)((int) sock, mask, (int) data);
}

////////////////////////////////////////////////////////////////////////////////

void socketCreateHandler(int sid, int mask, socketHandler_t handler, int arg)
{
	MprSocket		*sock;
	int			socketMask;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	socketMask = 0;
	if (mask & SOCKET_READABLE) {
		socketMask |= MPR_SOCKET_READABLE;
	}
	if (mask & SOCKET_WRITABLE) {
		socketMask |= MPR_SOCKET_WRITABLE;
	}
	if (mask & SOCKET_EXCEPTION) {
		socketMask |= MPR_SOCKET_WRITABLE;
	}
	sock->setCallback(socketIoWrapper, (void*) handler, (void*) arg, 
		socketMask);
}

////////////////////////////////////////////////////////////////////////////////

void socketDeleteHandler(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	sock->setCallback(0, 0, 0, 0);
}

////////////////////////////////////////////////////////////////////////////////

int socketEof(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->getEof() ? 1 : 0;
}

////////////////////////////////////////////////////////////////////////////////

int socketFlush(int sid)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int socketGetHandle(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->getFd();
}

////////////////////////////////////////////////////////////////////////////////

int socketRead(int sid, char *buf, int len)
{
	MprSocket		*sock;

	mprAssert(buf);
	mprAssert(len > 0);

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->read(buf, len);
}

////////////////////////////////////////////////////////////////////////////////

int socketWrite(int sid, char *buf, int len)
{
	MprSocket		*sock;

	mprAssert(buf);
	mprAssert(len > 0);

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->write(buf, len);
}

////////////////////////////////////////////////////////////////////////////////

int socketWriteString(int sid, char *buf)
{
	int		len;

	len = strlen(buf);
	return socketWrite(sid, buf, len);
}

////////////////////////////////////////////////////////////////////////////////

int socketSetBlock(int sid, int flags)
{
	MprSocket		*sock;
	int				old;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	old = sock->getBlockingMode() ? 1 : 0;
	sock->setBlockingMode(flags != 0 ? true : false);
	return old;
}

////////////////////////////////////////////////////////////////////////////////

int socketGetBlock(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->getBlockingMode() ? 1 : 0;
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// Misc ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//	Trace formats normally do not have new lines. 
//
void trace(int level, char *fmt, ...) 
{
	char	buf[MPR_MAX_LOG_STRING];
	va_list args;
	int		len;

	va_start(args, fmt);

	//
	//	mprVsprintf always ensures there is a null terminator. Subtract one
	//	for a new line
	//
	len = mprVsprintf(buf, sizeof(buf) - 1, fmt, args);
	buf[len++] = '\n';
	buf[len] = '\0';

	mprLog(level, buf);
	va_end(args);
}

////////////////////////////////////////////////////////////////////////////////

void traceRaw(char *buf) 
{
	mprLog(0, buf);
}

////////////////////////////////////////////////////////////////////////////////
//
//	Allocate a new file handle.  On the first call, the caller must set the
//	handle map to be a pointer to a null pointer.  *map points to the second
//	element in the handle array.
//

int hAlloc(void ***map)
{
	int		*mp;
	int		handle, len, memsize, incr;

	a_assert(map);

	if (*map == NULL) {
		incr = H_INCR;
		memsize = (incr + H_OFFSET) * sizeof(void**);
		if ((mp = (int*) balloc(B_L, memsize)) == NULL) {
			return -1;
		}
		memset(mp, 0, memsize);
		mp[H_LEN] = incr;
		mp[H_USED] = 0;
		*map = (void**) &mp[H_OFFSET];

	} else {
		mp = &((*(int**)map)[-H_OFFSET]);
	}

	len = mp[H_LEN];

	//
	//	Find the first null handle
	//
	if (mp[H_USED] < mp[H_LEN]) {
		for (handle = 0; handle < len; handle++) {
			if (mp[handle+H_OFFSET] == 0) {
				mp[H_USED]++;
				return handle;
			}
		}
	} else {
		handle = len;
	}

	//
	//	No free handle so grow the handle list. Grow list in chunks of H_INCR.
	//
	len += H_INCR;
	memsize = (len + H_OFFSET) * sizeof(void**);
	if ((mp = (int*) brealloc(B_L, (void*) mp, memsize)) == NULL) {
		return -1;
	}
	*map = (void**) &mp[H_OFFSET];
	mp[H_LEN] = len;
	memset(&mp[H_OFFSET + len - H_INCR], 0, sizeof(int*) * H_INCR);
	mp[H_USED]++;
	return handle;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Free a handle.  This function returns the value of the largest
//	handle in use plus 1, to be saved as a max value.
//

int hFree(void ***map, int handle)
{
	int		*mp;
	int		len;

	a_assert(map);
	mp = &((*(int**)map)[-H_OFFSET]);
	a_assert(mp[H_LEN] >= H_INCR);

	a_assert(mp[handle + H_OFFSET]);
	a_assert(mp[H_USED]);
	mp[handle + H_OFFSET] = 0;
	if (--(mp[H_USED]) == 0) {
		bfree(B_L, (void*) mp);
		*map = NULL;
	}

	//
	//	Find the greatest handle number in use.
	//
	if (*map == NULL) {
		handle = -1;
	} else {
		len = mp[H_LEN];
		if (mp[H_USED] < mp[H_LEN]) {
			for (handle = len - 1; handle >= 0; handle--) {
				if (mp[handle + H_OFFSET])
					break;
			}
		} else {
			handle = len;
		}
	}
	return handle + 1;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Allocate an entry in the halloc array.
//

int hAllocEntry(void ***list, int *max, int size)
{
	char	*cp;
	int		id;

	a_assert(list);
	a_assert(max);

	if ((id = hAlloc((void***) list)) < 0) {
		return -1;
	}

	if (size > 0) {
		if ((cp = (char*) balloc(B_L, size)) == NULL) {
			hFree(list, id);
			return -1;
		}
		a_assert(cp);
		memset(cp, 0, size);

		(*list)[id] = (void*) cp;
	}

	if (id >= *max) {
		*max = id + 1;
	}
	return id;
}

////////////////////////////////////////////////////////////////////////////////

int fmtAlloc(char **buf, int maxSize, char *fmt, ...)
{
	va_list		args;
	int			rc;

	va_start(args, fmt);	
	rc = mprAllocVsprintf(buf, maxSize, fmt, args);
	va_end(args);	
	return rc;
}

////////////////////////////////////////////////////////////////////////////////

int fmtValloc(char **buf, int maxSize, char *fmt, va_list arg)
{
	return mprAllocVsprintf(buf, maxSize, fmt, arg);
}

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////// Security Handler //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//	The GaCompatHandler is created for each incoming HTTP request. We tell
//	the Handler base class that we must run ALWAYS as a non-terminal
//	handler. Handlers be non-terminal where they can modify the request,
//	but not actually handle it.
//

GaCompatHandler::GaCompatHandler(char *extensions) :
	MaHandler("gaCompatHandler", extensions, MPR_HANDLER_ALWAYS)
{
	if (defaultServer == 0) {
		defaultServer = maGetHttp()->findServer("default");
		defaultHost = defaultServer->getDefaultHost();
	}
#if UNUSED
	if (defaultInterp == 0) {
		defaultInterp = ejsGetDefaultInterp();
	}
	if (defaultServer == 0 || defaultHost == 0 || defaultInterp == 0) {
		error(MPR_L, "Undefined server, host or EJS engine");
		return;
	}
#endif
}


////////////////////////////////////////////////////////////////////////////////
//
//	Destructor for the GaCompatHandler
//

GaCompatHandler::~GaCompatHandler()
{
}


////////////////////////////////////////////////////////////////////////////////
//
//	For maximum speed in servicing requests, we clone a pre-existing
//	handler when an incoming request arrives.
//

MaHandler *GaCompatHandler::cloneHandler()
{
	return new GaCompatHandler(extensions);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Called to see if this handler should process this request.
//	Return TRUE if you want this handler to match this request
//

int GaCompatHandler::matchRequest(MaRequest *rq, char *uri, int uriLen)
{
	//	Always match
	return 1;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Run the handler to service the request
//

int GaCompatHandler::run(MaRequest *rq)
{
	char  *uri;

	uri = rq->getUri();

	if (securityChecks(rq) < 0) {
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	}
	return 0;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Not yet debugged
//

int GaCompatHandler::securityChecks(MaRequest *rq)
{
#if FUTURE
	accessMeth_t	am;
	MaAuth			*auth;
	char			*type, *accessLimit, *requiredPassword;
	char			*uriPath, *ext, *mimeType, *query, *password, *user;
	char			*realm;
	int				flags, nRet, isSecure, authType;

	accessLimit = 0;
	realm = "UNKNOWN";

	auth = rq->getAuth();
	uriPath = maGetRequestUriPath(rq);
	ext = maGetRequestUriExt(rq);
	mimeType = maGetRequestMimeType(rq);
	query = maGetRequestUriQuery(rq);
	isSecure = maIsRequestSecure(rq);

	//
 	//	Get the critical request details
 	//
	password = maGetRequestPassword(rq);
	user = maGetRequestUserName(rq);

	//
	//	Get the access limit for the URL.  Exit if none found.
	//
	accessLimit = umGetAccessLimit((char*) uriPath);
	if (accessLimit == NULL) {
		return 0;
	}
		 
	//
	//	Check to see if URL must be encrypted
	//
#if BLD_FEATURE_SSL_MODULE
	nRet = umGetAccessLimitSecure(accessLimit);
	if (nRet && isSecure) {
		maSetResponseError(rq, 405, 
			"Access Denied\nSecure access is required.");
		trace(3, "SEC: Non-secure access attempted on <%s>\n", uriPath);
		bfree(B_L, accessLimit);
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	}
#endif

	//
	//	Get the access limit for the URL
	//
	am = umGetAccessMethodForURL(accessLimit);
	authType = (am == AM_DIGEST) ? MPR_HTTP_AUTH_DIGEST : MPR_HTTP_AUTH_BASIC;

	if (strcmp("127.0.0.1", maGetRequestIpAddr(rq)) == 0) {
		//
		//	Local access is always allowed
		//
		return 0;
	}

	if (am == AM_NONE) {
		//
		//	URL is supposed to be hidden!  Make like it wasn't found.
		//
		maSetResponseError(rq, 404, "Page Not Found");
		bfreeSafe(B_L, accessLimit);
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	} 

	requiredPassword = umGetUserPassword((char*) user);
	auth->setRequiredPassword();

	if (user && *user) {

		if (! umUserExists((char*) user)) {
			formatAuthResponse(rq, auth, realm, authType, 401, 
				"Access Denied\nUnknown User");
			trace(3, "SEC: Unknown user <%s> attempted to access <%s>\n", 
				user, uriPath);
			bfreeSafe(B_L, accessLimit);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}
		if (! umUserCanAccessURL((char*) user, accessLimit)) {
			formatAuthResponse(rq, auth, realm, authType, 403, 
				"Access Denied\nProhibited User");
			bfreeSafe(B_L, accessLimit);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}
		if (password && * password) {

			auth->setRequiredPassword();
			if (requiredPassword) {
				if (gstrcmp(password, requiredPassword) != 0) {
					maSetResponseError(rq, 401, 
						"Access Denied\nWrong Password");
					trace(3, "SEC: Password fail for user <%s>"
								"attempt to access <%s>\n", user, uriPath);
					bfreeSafe(B_L, accessLimit);
					return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
				}
				bfree (B_L, requiredPassword);
			}

#if BLD_FEATURE_DIGEST
		} else if (mprStrcmpAnyCase(maGetRequestAuthType(rq), "digest") == 0) {

			char *digestCalc;

			//
			//	Check digest for equivalence
			//
			requiredPassword = umGetUserPassword((char*) user);
			maSetRequestPassword(requiredPassword);
#if USING_AUTH_HANDLER_FOR_THIS
			a_assert(wp->digest);
			a_assert(wp->nonce);
							 
			digestCalc = websCalcDigest(wp);
			maCalcDigest(&requiredDigest, userName, requiredPassword, realm, 
				uri, nonce, qop, nc, 
				cnonce, rq->getMethod());
			mprFree(requiredPassword);
			a_assert(digestCalc);

			if (gstrcmp(wp->digest, digestCalc) != 0) {
				formatAuthResponse(rq, auth, realm, authType, 401, 
					"Access Denied\nWrong Password");
				bfreeSafe(B_L, accessLimit);
				return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
			}
			bfree (B_L, digestCalc);
#endif
#endif
		} else {
			//
			//	No password has been specified
			//
			formatAuthResponse(rq, auth, realm, authType, 401, 
				"Access to this document requires a password");
			bfreeSafe(B_L, accessLimit);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}

	} else if (am != AM_FULL) {
		//
		//	This will cause the browser to display a password / username dialog
		//
		formatAuthResponse(rq, auth, realm, authType, 401, 
			"Access to this document requires a User ID");
		bfreeSafe(B_L, accessLimit);
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	}
#endif
	return 0;
}


////////////////////////////////////////////////////////////////////////////////

void GaCompatHandler::formatAuthResponse(MaRequest *rq, MaAuth *auth, 
	char *realm, int authType, int code, char *userMsg)
{
#if BLD_FEATURE_DIGEST
	char		*nonceStr;
	char		*etag;
#endif
	char		*buf, *headers;

	rq->incAccessError();

	mprLog(3, "auth: formatAuthResponse: code %d, %s\n", code, userMsg);
	mprAllocSprintf(&buf, MPR_HTTP_BUFSIZE, 
		"<HTML><HEAD>\n<TITLE>Authentication Error: %s</TITLE>\n</HEAD>\r\n"
		"<BODY><H2>Authentication Error: %s</H2></BODY>\n</HTML>\r\n",
		userMsg, userMsg);

	if (authType == MPR_HTTP_AUTH_BASIC) {
		mprAllocSprintf(&headers, MPR_MAX_STRING,
			"WWW-Authenticate: Basic realm=\"%s\"", realm);

#if BLD_FEATURE_DIGEST
	} else if (authType == MPR_HTTP_AUTH_DIGEST) {

		//
		//	Use the etag as our opaque string
		//
		etag = rq->getEtag();
		if (etag == 0) {
			etag = "";
		}
		maCalcNonce(&nonceStr, rq->host->getSecret(), etag, realm);

		mprAllocSprintf(&headers, MPR_MAX_STRING,
			"WWW-Authenticate: Digest realm=\"%s\", domain=\"%s\", "
			"qop=\"auth\", nonce=\"%s\", opaque=\"%s\", algorithm=\"MD5\", "
			"stale=\"FALSE\"", 
			realm, rq->host->getName(), nonceStr, etag);

		mprFree(nonceStr);
#endif
	}
	rq->setHeader(headers);
	rq->formatAltResponse(code, buf, MPR_HTTP_DONT_ESCAPE);
	rq->flushOutput(MPR_HTTP_FOREGROUND_FLUSH, MPR_HTTP_FINISH_REQUEST);
	mprFree(headers);
	mprFree(buf);
}

////////////////////////////////////////////////////////////////////////////////

}	// extern "C"
#else
void mprCompatModuleDummy() {}

#endif // BLD_FEATURE_GACOMPAT_MODULE

//
// Local variables:
// tab-width: 4
// c-basic-offset: 4
// End:
// vim: sw=4 ts=4 
//
