///
///	@file 	gaCompatModule.cpp
/// @brief 	Compatibility module for the GoAhead Web Server
///	@overview Provide a reasonable measure of compatibility with the 
///		GoAhead WebServer APIs.
///	@remarks This module only supports single threaded operation without 
///	any support for virtual hosts.
//
/////////////////////////////////// Copyright //////////////////////////////////
//
//	@copy	default
//	
//	Copyright (c) Embedthis Software LLC, 2003-2009. All Rights Reserved.
//	
//	This software is distributed under commercial and open source licenses.
//	You may use the GPL open source license described below or you may acquire 
//	a commercial license from Embedthis Software. You agree to be fully bound 
//	by the terms of either license. Consult the LICENSE.TXT distributed with 
//	this software for full details.
//	
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version. See the GNU General Public License for more 
//	details at: http://www.mbedthis.com/downloads/gplLicense.html
//	
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	
//	This GPL license does NOT permit incorporating this software into 
//	proprietary programs. If you are unable to comply with the GPL, you must
//	acquire a commercial license to use this software. Commercial licenses 
//	for this software and support services are available from Embedthis 
//	Software at http://www.mbedthis.com 
//	
//	@end
//
////////////////////////////////// Includes ////////////////////////////////////

#include	"gaCompatModule.h"

//////////////////////////////////// Locals ////////////////////////////////////
#if BLD_FEATURE_GACOMPAT_MODULE

/* ASP KB config define*/
#include <pthread.h>

#define SERVER_IP "127.0.0.1"
#define SERVER_PORT 2280
#define MAXMESSAGELEN 10

int PrimaryUDP;
struct sockaddr_in ServerSock;
char sendbuf[MAXMESSAGELEN];
char recvbuf[MAXMESSAGELEN];
int uploadStatus = 0;
int upload_type = 0;
int file_length = 0;
int total_length = 0;
int upload_length = 0;
int folderType = -1;
int SladerWait = 0;

#define MAXBUFFLEN 2048
char *globalPostData = NULL;
int globalPostLen = 0;
char flobalFloderType[20];
char tmpFile[] = "/tmp/uploadx";

#define FILE_APPLICATION 1
#define FILE_FPGA 2
#define FILE_KERNEL 3
#define FILE_SILDERFW 4
#define FILE_FIRMWARE_ULTRAKEYPLUS 5
#define FILE_FIRMWARE_CLIENT 6
#define FILE_DYNAMIC_BUTTON 7
#define FILE_FS 8
#define FILE_BL 9


#define UPLOAD_STATUS_CONNTINUE 0
#define UPLOAD_STATUS_ERROR 2
#define UPLOAD_STATUS_OK 3
#define UPLOAD_STATUS_END 5

#define UPLOAD_PROCESS_TYPE_ERASE 1
#define UPLOAD_PROCESS_TYPE_WRITE 2
#define UPLOAD_PROCESS_TYPE_WAIT  3

#define FPGA_DEVICE "/dev/mtd6"
#define APPLICATION_DEVICE "/dev/mtd8"
#define KERNEL_DEVICE "/dev/mtd4"
#define FS_DEVICE "/dev/mtd5"

struct mtd_info_user {
	unsigned char type;
	unsigned int flags;
	unsigned int size;	 // Total size of the MTD
	unsigned int erasesize;
	unsigned int writesize;
	unsigned int oobsize;   // Amount of OOB data per block (e.g. 16)
	/* The below two fields are obsolete and broken, do not use them
	 * (TODO: remove at some point) */
	unsigned int ecctype;
	unsigned int eccsize;
};
typedef struct mtd_info_user mtd_info_t;

struct erase_info_user {
	unsigned int start;
	unsigned int length;
};
typedef struct erase_info_user erase_info_t;
#define MEMGETINFO              _IOR('M', 1, struct mtd_info_user)
#define MEMERASE                _IOW('M', 2, struct erase_info_user)

int mksock(int type)
{
    int sock = socket(AF_INET, type, 0);
    if (sock < 0)
    {
        printf("create socket error");
	 return -1;
    }
    return sock;
}

void BindSock(int sock)
{
    struct sockaddr_in sin;
    bzero(&sin,sizeof(struct sockaddr_in));  
    sin.sin_family = AF_INET;   
    sin.sin_addr.s_addr = INADDR_ANY;  
    sin.sin_port = 0; 
    
    bind(sock, (struct sockaddr*)&sin, sizeof(sin));
}
/***********************/

#define MPR_HTTP_MAX_GO_FORM		100

//
//	hAlloc list defines.
//
//	The handle list stores the length of the list and the number of used
//	handles in the first two words.  These are hidden from the caller by
//	returning a pointer to the third word to the caller
//

#define H_LEN		0		// First entry holds length of list
#define H_USED		1		// Second entry holds number of used
#define H_OFFSET	2		// Offset to real start of list 
#define H_INCR		16		// Grow handle list in chunks this size


class SymHashEntry : public MprHashEntry {
  public:
	sym_t			sym;
					SymHashEntry(char *key, value_t *vp);
	virtual			~SymHashEntry();
	value_t			*getValue() { return &sym.content; };
};


class WebsForm : public MaEgiForm {
  private:
	WebsFormCb		goFormCallback;

  public:
					WebsForm(char *formName, WebsFormCb fn);
					~WebsForm();
	void			run(MaRequest *rq, char *script, char *path, 
						char *query, char *postData, int postLen);
};

//
//	Single threaded
//
static MprFile 		*file;

static MaServer		*defaultServer;
static MaHost		*defaultHost;
static WebsForm		*websForms[MPR_HTTP_MAX_GO_FORM];
static int 			maxForm;

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// CompatModule /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int mprCompatInit(void *handle)
{
	if (maGetHttp() == 0) {
		return MPR_ERR_NOT_INITIALIZED;
	}
	new MaCompatModule(handle);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

typedef enum
{
	DISPLAY_SYS_MODE		= 0x00000001,
	CONFIG_SYSTEM			= 0x00000002,

	DISPLAY_DVR_TYPE		= 0x00000010,
	DISPLAY_DVR_USER		= 0x00000011,
	DISPLAY_BRIDGE_MODE		= 0x00000012,
	DISPLAY_BG_MASTER_IP	= 0x00000013,
	DISPLAY_DOME_NAME		= 0x00000014,
	DISPLAY_DOME_USED_ADDR	= 0x00000015,
	DISPLAY_DOME_TYPE		= 0x00000016,
	DISPLAY_DVR_IP			= 0x00000017,

	CONFIG_DVR_TYPE			= 0x00000110,
	CONFIG_DVR_USER			= 0x00000111,
	CONFIG_BRIDGE_MODE		= 0x00000112,
	CONFIG_BG_MASTER_IP		= 0x00000113,
	CONFIG_DOME_NAME		= 0x00000114,
	CONFIG_DOME_USED_ADDR	= 0x00000115,
	CONFIG_DOME_TYPE		= 0x00000116,	
	CONFIG_DVR_IP			= 0x00000117,

	DISPLAY_DVR_NO			= 0x00001000,
	DISPLAY_DVR_CHANNEL		= 0x00001001,

	DISPLAY_SERIAL_1		= 0x00010000,
	DISPLAY_SERIAL_2		= 0x00010001,
	CONFIG_SERIAL_1			= 0x00010011,
	CONFIG_SERIAL_2			= 0x00010111,
	CONFIG_SERIAL_APPLY		= 0x00011111,

	DISPLAY_IPADDRESS		= 0x00100000,
	DISPLAY_NETMASK			= 0x00100001,
	DISPLAY_GATEWAY			= 0x00100002,
	DISPLAY_DNS				= 0x00100003,
	SET_SYS_IPADDR			= 0x00110000,
	SET_SYS_NETMASK			= 0x00110001,
	SET_SYS_GATEWAY			= 0x00110002,
	SET_SYS_DNS				= 0x00110003,

	SYS_REBOOT				= 0x01000000,
	GET_SYS_VERSION			= 0x01000001,
	CHECK_USER				= 0x01000010,

	SWITCH_TO_FRAME			= 0x10000000
}ULTRA_KEY_TOUCH_TYPE;

void SendRequestToServer(int mode, int id, char *pSendRecvValue, unsigned long int kType)
{
	struct timeval tv;
    fd_set readfds;
	char sendbuf[MAXBUFFLEN] = "";
	char recvbuf[MAXBUFFLEN] = "";

    tv.tv_sec = 3;
    tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(PrimaryUDP, &readfds);

	socklen_t fromlen = sizeof(ServerSock);

	if(mode == 1) //get value
		sprintf(sendbuf, "%d:%d: :%08x", mode, id, kType);
	else //set value
		sprintf(sendbuf, "%d:%d:%s:%08x", mode, id, pSendRecvValue, kType);

	sendto(PrimaryUDP, (const char*)&sendbuf, strlen(sendbuf), 0, (const struct sockaddr*)&ServerSock,sizeof(ServerSock));

	if(mode == 0)//set
		return;

	if(select(PrimaryUDP+1, &readfds, NULL, NULL, &tv) > 0)
	{
		int len = recvfrom(PrimaryUDP, (char *)recvbuf, sizeof(recvbuf), 0, (struct sockaddr*)&ServerSock,&fromlen);

		if(mode == 1){
			memcpy(pSendRecvValue, recvbuf, len);
		}
	}
}

static int  aspDisplaySystemConfig(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	a_assert(websValid(wp));

	if (argc < 0)
	{
		return websWrite(wp, T(""));
	}

	if (strncmp("sysmode", argv[0], sizeof("sysmode")) == 0)
	{
		SendRequestToServer(1, 0, gValue, DISPLAY_SYS_MODE);
		if(gValue)
			return websWrite(wp, T("%d"), atoi(gValue));
		else
			return websWrite(wp, T(""));
	}
	else if (strncmp("syslanguage", argv[0], sizeof("syslanguage")) == 0)
	{
		return websWrite(wp, T("%d"), 0);
	}
	else
	{
		return websWrite(wp, T(""));
	}

	return websWrite(wp, T(""));
}

static void formConfigSystem(webs_t wp, char_t *path, char_t *query)
{
	char *systemMode = NULL;
	char *systemLanguage = NULL;
	char *systemBackground = NULL;

	systemMode = websGetVar(wp,"sysmode",T(""));
	systemLanguage = websGetVar(wp,"syslanguage",T(""));
	systemBackground = websGetVar(wp,"background",T(""));

	SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

	/* Call API to set system */
	SendRequestToServer(0, 0, systemMode, CONFIG_SYSTEM);

	/* set language */
#ifdef ULTRAKEYLITE

	switch(GetUltraKeyLiteOpMode())
	{
		case Mode_Standalone:
			//SwitchToFrame(UltraKeyLite_SysSreens+
			break;
		case Mode_VideBLX:
			//SwitchToFrame(
			break;
		case Mode_Maxpro:
			//SwitchToFrame(Sys_Config_Item_Menu+MAXPRO_NORMAL_MENU);

			break;
		default:
			//SwitchToFrame(UltraKeyLite_SysSreens+
			break;
	}
	SwitchToMode(GetUltraKeyLiteOpMode());
	formConfigSystemSetBackLight(systemBackground);
#endif

	websRedirect(wp, T("/ConfigSystem.asp"));
	return ;

}

static int  aspDisplayStandalone(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";
	a_assert(websValid(wp));

	if (argc < 0)
	{
		return websWrite(wp, T("%d"), 0);
	}

	if (strncmp("dvrtype", argv[0], sizeof("dvrtype")) == 0)
	{
		SendRequestToServer(1, 0, gValue, DISPLAY_DVR_TYPE);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp("dvruser", argv[0], sizeof("dvruser")) == 0)
	{
		SendRequestToServer(1, 0, gValue, DISPLAY_DVR_USER);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp("bridgemode", argv[0], sizeof("bridgemode")) == 0)
	{
		SendRequestToServer(1, 0, gValue, DISPLAY_BRIDGE_MODE);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strstr(argv[0], "bgmasterip") != NULL)
	{
		int id = 0;
		sscanf(argv[0], "bgmasterip%d", &id);

		SendRequestToServer(1, id, gValue, DISPLAY_BG_MASTER_IP);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp("domename", argv[0], sizeof("domename")) == 0)
	{
		SendRequestToServer(1, 0, gValue, DISPLAY_DOME_NAME);
		return websWrite(wp, T("%s"), gValue);
	}
	else if (strncmp("domeusedaddr", argv[0], sizeof("domeusedaddr")) == 0)
	{
		SendRequestToServer(1, 0, gValue, DISPLAY_DOME_USED_ADDR);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp("dometype", argv[0], sizeof("dometype")) == 0)
	{
    	SendRequestToServer(1, 0, gValue, DISPLAY_DOME_TYPE);
    	return websWrite(wp, T("%d"), atoi(gValue));
	}
	else
	{
		return websWrite(wp, T("%d"), 0);
	}

	return websWrite(wp, T("%d"), 0);
}

/* FIX: add hrep. 20110623 */
static int  aspDisplayDvrIp(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	SendRequestToServer(1, 0, gValue, DISPLAY_DVR_IP);

	return websWrite(wp, T("%s"), gValue);
}

static void formConfigStandalone(webs_t wp, char_t *path, char_t *query)
{
	char * dvrType = NULL;
	char * dvrUser = NULL;
	/* FIX: add hrep. 20110622 */
	char * dvrNum = NULL;
	char * dvrIp1 = NULL;
	char * dvrIp2 = NULL;
	char * dvrIp3 = NULL;
	char * dvrIp4 = NULL;
	//		
	char * bridgeMode = NULL;
	char * bgMasterIp1 = NULL;
	char * bgMasterIp2 = NULL;
	char * bgMasterIp3 = NULL;
	char * bgMasterIp4 = NULL;
	char * domeName = NULL;
	char * domeUsedAddr = NULL;
	char * domeType = NULL;
	char ip[4] ;
	/* FIX: add hrep. 20110622 */
	char hrepip[4] ;

	a_assert(websValid(wp));

	dvrType = websGetVar(wp,"dvrtype",T(""));
	dvrUser = websGetVar(wp,"dvruser",T(""));
	/* FIX: add hrep. 20110622 */
	dvrNum = websGetVar(wp,"dvrnum",T(""));	
	dvrIp1 = websGetVar(wp,"dvrip1",T(""));
    hrepip[0] = atoi(dvrIp1);
	dvrIp2 = websGetVar(wp,"dvrip2",T(""));
    hrepip[1] = atoi(dvrIp2);
	dvrIp3 = websGetVar(wp,"dvrip3",T(""));
    hrepip[2] = atoi(dvrIp3);
	dvrIp4 = websGetVar(wp,"dvrip4",T(""));
	hrepip[3] = atoi(dvrIp4);

	bridgeMode = websGetVar(wp,"bridgemode",T(""));
	bgMasterIp1 = websGetVar(wp,"bgmasterip1",T(""));
    ip[0] = atoi(bgMasterIp1);
	bgMasterIp2 = websGetVar(wp,"bgmasterip2",T(""));
    ip[1] = atoi(bgMasterIp2);
	bgMasterIp3 = websGetVar(wp,"bgmasterip3",T(""));
    ip[2] = atoi(bgMasterIp3);
	bgMasterIp4 = websGetVar(wp,"bgmasterip4",T(""));
    ip[3] = atoi(bgMasterIp4);

	domeName = websGetVar(wp,"domename",T(""));
	domeUsedAddr = websGetVar(wp,"domeusedaddr",T(""));
	domeType = websGetVar(wp,"dometype",T(""));

	/* FIX: add hrep. 20110622 */
	SendRequestToServer(0, (atoi(dvrNum)-1), hrepip, CONFIG_DVR_IP);
	SendRequestToServer(0, 0, bridgeMode, CONFIG_BRIDGE_MODE);
	//
//qianhao7
	//kbd_mode=atoi(bridgeMode);
//qianhao16
	if(atoi(bridgeMode) == 0)
		SendRequestToServer(0, 0, ip, CONFIG_BG_MASTER_IP);

	SendRequestToServer(0, 0, domeType, CONFIG_DOME_TYPE);
	SendRequestToServer(0, 0, domeUsedAddr, CONFIG_DOME_USED_ADDR);
	SendRequestToServer(0, 0, domeName, CONFIG_DOME_NAME);

	//qianhao12SetUltraKeyLiteDomeUsed(atoi(domeUsedAddr));
	SendRequestToServer(0, 0, dvrType, CONFIG_DVR_TYPE);

	websRedirect(wp, T("/Standalone.asp"));

	return ;

}

static int  aspDisplayDvrAllocationNO(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	SendRequestToServer(1, 0, gValue, DISPLAY_DVR_NO);

	return websWrite(wp, T("%s"), gValue);
}

static int  aspDisplayDvrAllocationChannel(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	SendRequestToServer(1, 0, gValue, DISPLAY_DVR_CHANNEL);

	return websWrite(wp, T("%s"), gValue);
}

static int  aspDisplayConfigSerialProc(unsigned long int kType, webs_t wp, char * name)
{
	char gValue[MAXBUFFLEN] = "";

	if (strncmp(name, "BaudRate", sizeof("BaudRate")) == 0)
	{
		SendRequestToServer(1, 1, gValue, kType);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp(name, "Parity", sizeof("Parity")) == 0)
	{
		SendRequestToServer(1, 2, gValue, kType);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp(name, "StopBits", sizeof("StopBits")) == 0)
	{
		SendRequestToServer(1, 3, gValue, kType);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp(name, "DataBits", sizeof("DataBits")) == 0)
	{
		SendRequestToServer(1, 4, gValue, kType);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else if (strncmp(name, "SerialMode", sizeof("SerialMode")) == 0)
	{
		SendRequestToServer(1, 5, gValue, kType);
		return websWrite(wp, T("%d"), atoi(gValue));
	}
	else
	{
		return websWrite(wp, T("%d"), "0");
	}

	return websWrite(wp, T("%d"), "0");
}

static int  aspDisplayConfigSerial(int eid, webs_t wp, int argc, char_t **argv)
{
	a_assert(websValid(wp));
	unsigned long int type;

	if (argc < 2)
	{
		return websWrite(wp, T("%d"), "0");
	}

	if (strncmp("port1", argv[0], sizeof("port1")) == 0)
	{
		type = DISPLAY_SERIAL_1;
	}
	else if (strncmp("port2", argv[0], sizeof("port1")) == 0)
	{
		type = DISPLAY_SERIAL_2;
	}
	else
	{
		return websWrite(wp, T("%d"), "0");
	}

	/* Call API to display Serial  Config */
	return aspDisplayConfigSerialProc(type, wp, argv[1]);
}

static void getSerialConfig(webs_t wp,int number,char** baudRate,char** parity,char** stopBit,char** dataBit,char** serialMode)
{
	char variableName[50] = "";

	sprintf(variableName,"hBaudRate%d",number);
	*baudRate = websGetVar(wp,variableName,T(""));
	sprintf(variableName,"hParity%d",number);
	*parity = websGetVar(wp,variableName,T(""));
	sprintf(variableName,"hStopBits%d",number);
	*stopBit = websGetVar(wp,variableName,T(""));
	sprintf(variableName,"hDataBits%d",number);
	*dataBit = websGetVar(wp,variableName,T(""));
	sprintf(variableName,"hSerialMode%d",number);
	*serialMode = websGetVar(wp,variableName,T(""));
	return ;
}

static void formConfigSerialPort(webs_t wp, char_t *path, char_t *query)
{
	char* baudRate[2];
	char* parity[2];
	char* stopBit[2];
	char* dataBit[2];
	char* serialMode[2];
	int i = -1;
	unsigned long int type;

	a_assert(websValid(wp));
	memset(baudRate, 0, 2 * sizeof(char *));
	memset(parity, 0, 2 * sizeof(char *));
	memset(stopBit, 0, 2 * sizeof(char *));
	memset(dataBit, 0, 2 * sizeof(char *));
	memset(serialMode, 0, 2 * sizeof(char *));

	SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

	for (i = 0; i < 2; i ++)
	{
		getSerialConfig(wp,i + 1, &baudRate[i], &parity[i], &stopBit[i], &dataBit[i], &serialMode[i]);
		//printf("baudRate:%s parity:%s stopBit:%s dataBit:%s serialMode:%s\n", baudRate[i], parity[i], stopBit[i], dataBit[i], serialMode[i]);
	}

	for (i = 0; i < 2; i ++)
	{
		char applyID[2] = "";

		if(i == 0)
			type = CONFIG_SERIAL_1;
		else
			type = CONFIG_SERIAL_2;
		sprintf(applyID, "%d", i);

		SendRequestToServer(0, 1, baudRate[i], type);
		SendRequestToServer(0, 2, parity[i], type);
		SendRequestToServer(0, 3, stopBit[i], type);
		SendRequestToServer(0, 4, dataBit[i], type);
		SendRequestToServer(0, 5, serialMode[i], type);

		SendRequestToServer(0, 0, applyID, CONFIG_SERIAL_APPLY);
	}

	return websRedirect(wp, T("/ConfigSerialPort.asp"));

	if ((strcmp(serialMode[0], "485") == 0) && (strcmp(serialMode[1], "485") == 0))
	{
		for (i = 0; i < 2; i ++)
		{
			char applyID[2] = "";

			if(i == 0)
				type = CONFIG_SERIAL_1;
			else
				type = CONFIG_SERIAL_2;
			sprintf(applyID, "%d", i);

			SendRequestToServer(0, 1, baudRate[i], type);
			SendRequestToServer(0, 2, parity[i], type);
			SendRequestToServer(0, 3, stopBit[i], type);
			SendRequestToServer(0, 4, dataBit[i], type);
			SendRequestToServer(0, 5, serialMode[i], type);

			SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

			SendRequestToServer(0, 0, applyID, CONFIG_SERIAL_APPLY);
		}

		return websRedirect(wp, T("/ConfigSerialPort.asp"));
	}

	if (strcmp(serialMode[0], "422") == 0)
	{
		SendRequestToServer(0, 1, baudRate[0], CONFIG_SERIAL_1);
		SendRequestToServer(0, 2, parity[0], CONFIG_SERIAL_1);
		SendRequestToServer(0, 3, stopBit[0], CONFIG_SERIAL_1);
		SendRequestToServer(0, 4, dataBit[0], CONFIG_SERIAL_1);
		SendRequestToServer(0, 5, serialMode[0], CONFIG_SERIAL_1);

		SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

		SendRequestToServer(0, 0, "0", CONFIG_SERIAL_APPLY);

		return websRedirect(wp, T("/ConfigSerialPort.asp"));
	}

	if (strcmp(serialMode[1], "232") == 0)
	{
		SendRequestToServer(0, 1, baudRate[1], CONFIG_SERIAL_2);
		SendRequestToServer(0, 2, parity[1], CONFIG_SERIAL_2);
		SendRequestToServer(0, 3, stopBit[1], CONFIG_SERIAL_2);
		SendRequestToServer(0, 4, dataBit[1], CONFIG_SERIAL_2);
		SendRequestToServer(0, 5, serialMode[1], CONFIG_SERIAL_2);

		SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

		SendRequestToServer(0, 0, "1", CONFIG_SERIAL_APPLY);

		return websRedirect(wp, T("/ConfigSerialPort.asp"));
	}

	return websRedirect(wp, T("/ConfigSerialPort.asp"));
}

static int  aspDisplayIPAddr(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	a_assert(websValid(wp));

	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	SendRequestToServer(1, atoi(argv[0]), gValue, DISPLAY_IPADDRESS);

	return websWrite(wp, T("%d"), atoi(gValue));
}

static int  aspDisplayNetmask(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	a_assert(websValid(wp));

	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	SendRequestToServer(1, atoi(argv[0]), gValue, DISPLAY_NETMASK);

	return websWrite(wp, T("%d"), atoi(gValue));
}

static int  aspDisplayGateWay(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	a_assert(websValid(wp));

	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	SendRequestToServer(1, atoi(argv[0]), gValue, DISPLAY_GATEWAY);

	return websWrite(wp, T("%d"), atoi(gValue));
}

static int  aspDisplayDns(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";

	a_assert(websValid(wp));

	if (argc < 1)
	{
		return websWrite(wp, T("%d"), "0");
	}

	SendRequestToServer(1, atoi(argv[0]), gValue, DISPLAY_DNS);

	return websWrite(wp, T("%d"), atoi(gValue));
}

static int CheckIP(char * mark,int validnum)
{
	int i;
	int intValue = 0;
	for(i = 0; i < strlen(mark); i++)
	{
		if(mark[i] == '\n' || mark[i] == '\r')
		{
			mark[i] = 0;
		}
		else
		{
			if(mark[i] > '9'||mark[i] < '0')
				return -1;
		}
	}
	if(strlen(mark) > 3 || strlen(mark) <= 0)
		return -1;
	for(i = 0; i < strlen(mark); i++)
	{
		intValue = intValue * 10 + (mark[i] - '0');
		if(intValue > validnum) return -1;
	}

	return intValue;
}

static int getip(webs_t wp,char* var, int iptype[],int validnum)
{
	int i;
	char* sp;
	char varName[100];

	for(i = 0; i < 4; i++)
	{
		sprintf(varName, "%s%d",var, i + 1);
		sp = websGetVar(wp,varName, T(""));
		if(*sp == 0){
			return -1;
		}
		if(i == 0)
			iptype[i] = CheckIP(sp,validnum);
		else
			iptype[i] = CheckIP(sp,255);
		if(iptype[i] < 0){
			return -1;
		}
	}

	return 1;
}

static void formIPConfig(webs_t wp, char_t *path, char_t *query)
{
	int netaddress[4];
	int netmask[4];
	int gateway[4];
	int DNS[4];
	int i = -1;

	char chAddr[4];
	char chNetmask[4];
	char chGateway[4];
	char chDNS[4];

	memset(netaddress, 0, 4 * sizeof(int));
	memset(netmask, 0, 4 * sizeof(int));
	memset(gateway, 0, 4 * sizeof(int));
	memset(DNS, 0, 4 * sizeof(int));

	memset(chAddr, 0, 4 * sizeof(char));
	memset(chNetmask, 0, 4 * sizeof(char));
	memset(chGateway, 0, 4 * sizeof(char));
	memset(chDNS, 0, 4 * sizeof(char));

	if(getip(wp,"ipaddr", netaddress, 239) < 0)
	{
		websRedirect(wp, T("/ConfigDispIPError.asp"));
		websDone(wp, 200);
		return;
	}

	if(getip(wp,"submsk",netmask, 255) < 0)
	{
		websRedirect(wp, T("/ConfigDispIPError.asp"));
		websDone(wp, 200);
		return;
	}

	if(getip(wp,"getway", gateway, 239) < 0)
	{
		websRedirect(wp, T("/ConfigDispIPError.asp"));
		websDone(wp, 200);
		return;
	}

	if(getip(wp,"dns", DNS, 239) < 0)
	{
		websRedirect(wp, T("/ConfigDispIPError.asp"));
		websDone(wp, 200);
		return;
	}

	for (i = 0; i < 4; i ++)
	{
		chAddr[i] = netaddress[i];
		chNetmask[i] = netmask[i];
		chGateway[i] = gateway[i];
		chDNS[i] = DNS[i];
	}

	SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

	SendRequestToServer(0, 0, chAddr, SET_SYS_IPADDR);
	SendRequestToServer(0, 0, chNetmask, SET_SYS_NETMASK);
	SendRequestToServer(0, 0, chGateway, SET_SYS_GATEWAY);
	SendRequestToServer(0, 0, chDNS, SET_SYS_DNS);

	websDone(wp, 200);
	return;
}

static void formReboot(webs_t wp, char_t *path, char_t *query)
{
	websRedirect(wp, T("/reboot.htm"));

	SendRequestToServer(0, 0, "NULL", SYS_REBOOT);

	return ;
}

static int printError(webs_t wp,int errorNumber)
{
	switch (errorNumber)
	{
		case 1:
			websWrite(wp,T(""));
			break;
		case 2:
			websWrite(wp,T("Error: Fail to change password."));
			break;
		case 3:
			websWrite(wp,T("Error: Incorrect old password."));
			break;
		case 4:
			websWrite(wp,T("Error: New password should not be empty."));
			break;
		case 5:
			websWrite(wp,T("Error: New password is too long."));
			break;
		case 6:
			websWrite(wp,T("Error: New passwords are not the same."));
			break;
		case 7:
			websWrite(wp,T("Error: User password saving error."));
			break;
		case 8:
			websWrite(wp,T("Change password success."));
			break;
		case 9:
			websWrite(wp,T("Error: Can't change password."));
			break;
		case 10:
			websWrite(wp,T("Error: Can't delete user."));
			break;
		case 11:
			websWrite(wp,T("Delete  user success."));
			break;
		case 12:
			websWrite(wp,T("Error: Have the same user."));
			break;
		case 13:
			websWrite(wp,T("Error: Can't add user."));
			break;
		case 14:
			websWrite(wp,T("Add user success."));
			break;
		/* FIX: only A-Z a-z and 0-9 can be used as password. 20110303 */
		case 15:
			websWrite(wp,T("Error: only A-Z, a-z and 0-9 can be used as password."));
			break;
		default:
			websWrite(wp,T(""));
			break;
	}
	return 0;
}

static int changePassworkError = 0;
static int  aspCheckChangePass(int eid, webs_t wp, int argc, char_t **argv)
{
	printError(wp, changePassworkError);
	changePassworkError = 0;
	return 0;
}

static void formChangePassword(webs_t wp, char_t *path, char_t *query)
{
	char_t* oldpassword;
	char_t* newpassword;
	char_t* newpassword1;

	oldpassword = websGetVar(wp,"OldPass",T(""));
	newpassword = websGetVar(wp,"NewPass",T(""));
	newpassword1 = websGetVar(wp,"ConfirmPass",T(""));

	if(*oldpassword == 0 && *newpassword == 0 && *newpassword1 == 0)
	{
		changePassworkError = 1;
		return websRedirect(wp, T("/ChangePass.asp"));
	}

	if(*oldpassword == 0 || strlen(oldpassword) > 30)
	{
		changePassworkError = 3;//Incorrect Old Password
		return websRedirect(wp, T("/ChangePass.asp"));
	}

	if(*newpassword == 0)
	{
		changePassworkError = 4;//new password empty
		return websRedirect(wp, T("/ChangePass.asp"));
	}

	if(strlen(newpassword) > 30)
	{
		changePassworkError = 5;//new password too long
		return websRedirect(wp, T("/ChangePass.asp"));
	}

	/* FIX: only A-Z a-z and 0-9 can be used as password. 20110303 */
	char ii=0;
	for(;*(newpassword+ii)!='\0';ii++)
	{
		if(!((*(newpassword+ii)>='A' && *(newpassword+ii)<='Z') ||
				(*(newpassword+ii)>='a' && *(newpassword+ii)<='z') ||
				(*(newpassword+ii)>='0' && *(newpassword+ii)<='9')))
		{
			changePassworkError = 15;
			return websRedirect(wp, T("/ChangePass.asp"));
		}
	}

	if (umUserExists("Administrator"))
	{
		char_t * userpass = umGetUserPassword("Administrator");

		if (userpass)
		{
			if (gstrcmp(oldpassword, userpass) != 0)
			{
				changePassworkError = 3;//Incorrect Old Password
				return websRedirect(wp, T("/ChangePass.asp"));
			}
			bfree (B_L, userpass);
		}
	}

	if(*newpassword1 == 0)
	{
		changePassworkError = 6;//Passwords are not the same
		return websRedirect(wp, T("/ChangePass.asp"));
	}

	if (gstrcmp(newpassword, newpassword1) != 0)
	{
		changePassworkError = 6;//Passwords are not the same
		return websRedirect(wp, T("/ChangePass.asp"));
	}

	umDeleteUser("Administrator");
	umAddUser("Administrator",newpassword, "administrator", 0, FALSE);

	if(umCommit(NULL) != 0)
	{
		changePassworkError = 7;//User Password save error.
	}
	else
	{
		changePassworkError = 8;//Password change success.
	}

	return websRedirect(wp, T("/ChangePass.asp"));
}

static int  aspShowVersion(int eid, webs_t wp, int argc, char_t **argv)
{
	char gValue[MAXBUFFLEN] = "";
	SendRequestToServer(1, 0, gValue, GET_SYS_VERSION);

	return websWrite(wp, T("%s"), gValue);
}

int aspCheckUser(int eid, webs_t wp, int argc, char_t **argv){
	char_t* username;
	char_t* password;
	char gValue[MAXBUFFLEN] = "";

	username = websGetVar(wp,"UserName",T(""));
	password = websGetVar(wp,"Authentication",T(""));

	if(password == NULL || *password == 0){
		return websWrite(wp,T(""));
	}

	if(username == NULL || *username == 0){
		return websWrite(wp,T(""));
	}

	if (!umUserExists(username)) {
		websWrite(wp,T(""));
		return 0;
	}
	if (password && * password) {
		char_t * userpass = umGetUserPassword(username);
		if (userpass) {
			if (gstrcmp(password, userpass) != 0) {
				return websWrite(wp,T(""));
			}
			bfree (B_L, userpass);
		}
		else {
			return websWrite(wp,T(""));
		}
	}

	//SendRequestToServer(1, 0, gValue, CHECK_USER);
	//return websWrite(wp, T("%s"), gValue);
	return websWrite(wp, T("%s"), username);
}

static int check_filename(char* str, char* folder)
{
	char* p;
	char* file_ext = NULL;
	char* file_type = NULL;
	int calc_time = 0;
	int dot_calc = 0;
	int str_length;
	str_length = strlen(str);
	p = str + str_length;

	for(; p != str - 1; p--)
	{
		if((int) *p == 13 || (int) *p == 10 || *p == '\"')
			*p = 0;

		if(*p == '.')
		{
			dot_calc ++;
			if(dot_calc == 1)
			{
				*p = 0;
				file_ext = p + 1;
			}
		}
		if(*p == '_')
		{
			*p = 0;
			calc_time ++;
			if(calc_time == 1)
			{
				file_type = p+1;
			}
		}
		if(*p == '/' || *p == '\\')
		{
			break;
		}
	}

	if(strstr(folder,"Application")!= NULL)
	{
		if((strcmp(file_ext,"jffs2") != 0) ||((strcmp(file_type,"App") != 0) && (strcmp(file_type,"app") != 0) && (strcmp(file_type,"APP") != 0)))
		{
			return -1;
		}
		return FILE_APPLICATION;
	}
	if(strstr(folder,"FPGA")!= NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||((strcmp(file_type,"FPGA") != 0) && (strcmp(file_type,"fpga") != 0) && (strcmp(file_type,"Fpga") != 0)))
		{

			return -1;
		}
		return FILE_FPGA;
	}
	if(strstr(folder,"Kernel")!=NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||( (strcmp(file_type,"KERNEL") != 0) && (strcmp(file_type,"kernel") != 0) && (strcmp(file_type,"Kernel") != 0)))
		{
			return -1;
		}
		return FILE_KERNEL;
	}
	if(strstr(folder,"FileSystem")!= NULL)
	{
		if((strcmp(file_ext,"jffs2") != 0) ||((strcmp(file_type,"FS") != 0) && (strcmp(file_type,"Fs") != 0) && (strcmp(file_type,"fs") != 0)))
		{
			return -1;
		}
		return FILE_FS;
	}

	if(strstr(folder,"BL")!= NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||((strcmp(file_type,"Loader") != 0) && (strcmp(file_type,"loader") != 0) && (strcmp(file_type,"LOADER") != 0)))
		{
			return -1;
		}
		return FILE_BL;
	}
	if(strstr(folder,"SliderFW")!=NULL)
	{
		if((strcmp(file_ext,"bin") != 0) ||( (strcmp(file_type,"SLIDERFW") != 0) && (strcmp(file_type,"sliderFW") != 0) && (strcmp(file_type,"SliderFW") != 0)))
		{
			return -1;
		}

		return FILE_SILDERFW;
	}

	if(strstr(folder,"UltraKeyPlusFW")!=NULL)
	{
		return FILE_FIRMWARE_ULTRAKEYPLUS;
	}
	if(strstr(folder,"ClientFW")!=NULL)
	{
		return FILE_FIRMWARE_CLIENT;
	}
	if(strstr(folder,"configuration")!=NULL)
	{
		return FILE_DYNAMIC_BUTTON;
	}
	return -1;
}

static int mtd_burn_image(int file_length_write, char *mtd_device, FILE* fp)
{
	int device_fd = -1;
	unsigned char  *tmp_buf = NULL;
	long buf_size = 4 * 1024;
	long image_size = 0;
	mtd_info_t meminfo;
	erase_info_t erase;

	memset(&meminfo, 0, sizeof(meminfo));
	memset(&erase, 0, sizeof(erase));
	tmp_buf = (unsigned char *)malloc(buf_size);
	if(tmp_buf == NULL)
	{
		return -1;
	}
	memset(tmp_buf, 0, buf_size);
	device_fd = open(mtd_device, O_RDWR | O_SYNC);
	if(device_fd < 0)
	{
		free(tmp_buf);
		return -1;
	}

	//get the information of the image
	image_size = file_length_write;
	// Erase the mtd device first
	if(ioctl(device_fd, MEMGETINFO, &meminfo) != 0)
	{
		return -1;
	}
	erase.length = meminfo.erasesize;
	file_length = meminfo.size;
	//printf("meminfo.erasesize=%x.................meminfo.size=%x\n",meminfo.erasesize,meminfo.size);

	upload_type  = UPLOAD_PROCESS_TYPE_ERASE;
	for (erase.start = 0; erase.start < meminfo.size;erase.start += meminfo.erasesize)
	{
		upload_length = erase.start;
		if(ioctl(device_fd, MEMERASE, &erase) != 0)
		{
			return -1;
		}
	}

	//Write MTD device
	upload_type = UPLOAD_PROCESS_TYPE_WRITE;

	file_length = image_size;
	upload_length = file_length - image_size;

	while(image_size > 0)
	{
		int tmp,ret;

		tmp=fread(tmp_buf, 1, buf_size, fp);
		if(tmp <=0)
		{
			free(tmp_buf);
			printf("%s, %d, tmp=%x pos=%x image_size=%x\n", __FILE__, __LINE__,tmp,ftell(fp),image_size);
			fclose(fp);
			close(device_fd);
			return -1;
		}
		ret=write(device_fd, tmp_buf, tmp);
		if (ret!=tmp)
		{
			free(tmp_buf);
			printf("%s, %d, tmp=%x ret=%x pos=%x image_size=%x\n", __FILE__, __LINE__,tmp,ret,ftell(fp),image_size);
			fclose(fp);
			close(device_fd);
			return -1;
		}
		image_size -= tmp;

		upload_length = file_length - image_size;
	}
	free(tmp_buf);
	close(device_fd);
	return 0;
}

void *write_flash(void *arg)
{
	FILE *       fp;
	char buf[512] = " ";
	char folder[50] = " ";
	char * point = NULL;
	int delNum = 0;
	int fileLength = -1;

	total_length = globalPostLen;

	fp = fopen(tmpFile,"r");
	if(fp == NULL)
	{
		//file open error
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);
		return 0;
	}
	// first line is MIME boundary
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum = strlen(buf) * 3 + 4;
	// second line contains "Content-Disposition
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(buf);
	// third line is the enter
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(buf);
	//fourth line is the folder name
	if (fgets(folder, sizeof(folder), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(folder);
	if ((point=strchr(folder, '\r')) != NULL)
	{
		*point = '\0';
	}
	if ((point=strchr(folder, '\n')) != NULL)
	{
		*point = '\0';
	}

	//fifth line is the boundary
	if (fgets(&buf[2], sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	// sixth line is Content-Disposition and contain file name:
	if (fgets(buf, sizeof(buf), fp) == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	delNum += strlen(buf);
	point = strstr(buf, "filename=");
	if (point == NULL)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	//check file name and folder name
	point += strlen("filename=");
	if((folderType = check_filename(point,folder)) == -1)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	while (fgets(buf, sizeof(buf), fp) != NULL)
	{
		delNum += strlen(buf);
		if (strlen(buf) <= 2)
			break;
	}
	if(total_length < 0)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}
	fileLength = total_length - delNum;

	if(fileLength <= 0)
	{
		uploadStatus = UPLOAD_STATUS_ERROR;
		fclose(fp);
		unlink(tmpFile);

		return 0;
	}

	if(folderType == FILE_APPLICATION)
	{
		if(mtd_burn_image(fileLength,APPLICATION_DEVICE,fp) < 0)
		{
			uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
		}
		else
		{
			uploadStatus = UPLOAD_STATUS_OK;
		}
	}
	else if(folderType == FILE_FPGA)
	{
		if(mtd_burn_image(fileLength,FPGA_DEVICE,fp) < 0)
		{
			uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
		}
		else
		{
			uploadStatus = UPLOAD_STATUS_OK;
		}
	}
	else if(folderType == FILE_KERNEL)
	{
		if(mtd_burn_image(fileLength,KERNEL_DEVICE,fp) < 0)
		{
			uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
		}
		else
		{
			uploadStatus = UPLOAD_STATUS_OK;
		}
	}
    else if(folderType == FILE_FS)
    {
    	if(mtd_burn_image(fileLength,FS_DEVICE,fp) < 0)
        {
        	uploadStatus = UPLOAD_STATUS_ERROR;// write mtd error
        }
        else
        {
            uploadStatus = UPLOAD_STATUS_OK;
        }
    }    
	else
	{
		//should not come here
		uploadStatus = UPLOAD_STATUS_ERROR;
	}
	//close and delete the tmp file.
	if(fp != NULL)
	{
		fclose(fp);
		fp = NULL;
	}
	unlink(tmpFile);
	return 0;
}

static void formUpload(webs_t wp, char_t *path, char_t *query)
{
	a_assert(websValid(wp));

	FILE *fp = fopen(tmpFile, "wb+");
	if (fp == NULL)
		return;

	if (fwrite(globalPostData, 1, globalPostLen, fp) != globalPostLen)
		return;

	fclose(fp);

	uploadStatus = UPLOAD_STATUS_CONNTINUE;

	websRedirect(wp, T("/uploadstatus.asp"));

	SendRequestToServer(0, 0, "1", SWITCH_TO_FRAME);

	pthread_t thread;
	pthread_create(&thread, 0, write_flash, NULL);
	pthread_detach(thread);

	return;
}

void formUploadStatus(webs_t wp, char_t *path, char_t *query)
{
	websRedirect(wp, T("/uploadstatus.asp"));
	return ;
}

int  aspJump(int eid, webs_t wp, int argc, char_t **argv)
{
	/*
	 * uploadStatus = UPLOAD_STATUS_CONNTINUE :not wrong at present, need jump
	 * uploadStatus = UPLOAD_STATUS_OK :successful
	 */
	if(uploadStatus == UPLOAD_STATUS_CONNTINUE)
	{
		websWrite(wp,T("<meta http-equiv=\"refresh\" content=\"3; url=/uploadstatus.asp\">"));
	}
	else
	{
		websWrite(wp,T(""));
	}

	return 1;
}

int  aspShowProgress(int eid, webs_t wp, int argc, char_t **argv)
{
	/*
	 * uploadStatus = UPLOAD_STATUS_CONNTINUE :not wrong at present, need jump
	 * uploadStatus = UPLOAD_STATUS_OK :successful
	 */
	int percent = 0;
	int i = 0;

	SladerWait ++;
	if(uploadStatus == UPLOAD_STATUS_CONNTINUE)
	{
		if(file_length == 0)
		{
			percent = 0;
		}
		else
		{
			percent = (upload_length * 100) / file_length;
		}
		if(upload_type  == UPLOAD_PROCESS_TYPE_ERASE)
		{
			websWrite(wp,T("Erasing old files: %d %c"), percent,'%');
		}
		else if(upload_type  == UPLOAD_PROCESS_TYPE_WRITE)
		{
			websWrite(wp,T("Copying new files: %d %c"), percent,'%');
		}
		else if(upload_type  == UPLOAD_PROCESS_TYPE_WAIT)
		{
			websWrite(wp,T("Please wait "));
			for (i = 0; i <= SladerWait; i ++)
			{
				websWrite(wp,T("."));
			}
		}
	}
	else
	{
		if(uploadStatus == UPLOAD_STATUS_ERROR)
		{
			uploadStatus = UPLOAD_STATUS_END;
			websWrite(wp, T("Error: Upgrade failed."));
		}
		if(uploadStatus == UPLOAD_STATUS_OK)
		{
			uploadStatus = UPLOAD_STATUS_END;
			/* 
			 * FIX: use different messages for kernel/fs/app upgrading 
			 * per Sean's beta feedback. 
			 * 20110407
			 */
			//websWrite(wp,T("Upgrade success: Please Power Off UltraKeyPlus and then Power On!"));
			if(FILE_APPLICATION==folderType)
			{
				websWrite(wp,T("Upgrade success: System will reboot automatically, Please wait..."));
			}
			else if(FILE_KERNEL==folderType)
			{
				websWrite(wp,T("Upgrade success: System will reboot automatically, Please wait..."));
			}
			else if(FILE_FS==folderType)
			{
				websWrite(wp,T("Upgrade success: Please Power Off UltraKeyTouch and then Power On!"));
			}
			//wp->rebootFlag = REBOOTFLAG;
			system("reboot");
		}
		websWrite(wp,T(""));
	}

	return 1;
}

void parsePostData(char *postData, int postLen)
{
	char *start = NULL;
	char multiPartBoundary[256] = "";
	int len = 0;
	char *ftype = NULL;

	start = postData;
	len = postLen;

	if( (ftype = strstr(postData, "name=\"folders\"")) )
	{
		int i = 0;

		ftype += 18; //name=""\r\n\r\n
		while(*ftype != 0x0d)
		{
			flobalFloderType[i++] = *ftype++;
			flobalFloderType[i] = '\0';
		}
	}
	else
		memset(flobalFloderType, 0, 20);

	if((start = strstr(postData, "Content-Disposition: form-data")) == 0)
	{
		globalPostData = NULL;
		globalPostLen = 0;
		return;
	}

	memcpy(multiPartBoundary, postData, start - postData - 2);

	if((start = strstr(postData, "Content-Type:")) == 0)
	{
		globalPostData = NULL;
		globalPostLen = 0;
		return;
	}

	while( !(*start == '\r' && *(start+1) == '\n') )
	{
		start++;
	}
	start+=4; //two \r\n
	len -= (start - postData);

	globalPostData = start;
	globalPostLen = len - strlen(multiPartBoundary) - 2 - 4; //--\r\n
}

////////////////////////////////////////////////////////////////////////////////

MaCompatModule::MaCompatModule(void *handle) : MaModule("gaCompat", handle)
{
	file = new MprFile();

	umOpen();
	umRestore(T("umconfig.txt"));

	/* Asp Define */
	websAspDefine(T("DisplaySystemConfig"), aspDisplaySystemConfig);
	websAspDefine(T("DisplayStandalone"), aspDisplayStandalone);
	websAspDefine(T("DisplayDvrIp"), aspDisplayDvrIp);
	websAspDefine(T("DisplayAllocationNO"), aspDisplayDvrAllocationNO);
	websAspDefine(T("DisplayAllocationChannel"), aspDisplayDvrAllocationChannel);
	websAspDefine(T("DisplayConfigSerial"), aspDisplayConfigSerial);
	websAspDefine(T("DisplayIPAddr"), aspDisplayIPAddr);
	websAspDefine(T("DisplayNetmask"), aspDisplayNetmask);
	websAspDefine(T("DisplayGateWay"), aspDisplayGateWay);
	websAspDefine(T("DisplayDns"), aspDisplayDns);
	websAspDefine(T("checkChangePassword"), aspCheckChangePass);
	websAspDefine(T("ShowVersion"), aspShowVersion);
	websAspDefine(T("checkUser"), aspCheckUser);
	websAspDefine(T("ShowProgress"), aspShowProgress);
	websAspDefine(T("Jump"), aspJump);

	websFormDefine(T("ConfigSystem"), formConfigSystem);
	websFormDefine(T("ConfigStandalone"), formConfigStandalone);
	websFormDefine(T("ConfigSerialPort"), formConfigSerialPort);
	websFormDefine(T("IPConfig"), formIPConfig);
	websFormDefine(T("Reboot"), formReboot);
	websFormDefine(T("ChangePassword"), formChangePassword);
	websFormDefine(T("Upload"), formUpload);
	websFormDefine(T("UploadStatus"), formUploadStatus);

	/* ASP local socket set */
	PrimaryUDP = mksock(SOCK_DGRAM);
	BindSock(PrimaryUDP);
	bzero(&ServerSock,sizeof(struct sockaddr_in));
	ServerSock.sin_family = AF_INET;
	ServerSock.sin_addr.s_addr = inet_addr(SERVER_IP);
	ServerSock.sin_port = htons(SERVER_PORT);
}

////////////////////////////////////////////////////////////////////////////////

MaCompatModule::~MaCompatModule()
{
	//
	//	No need to delete websForms[]. EgiHandler will delete these for us
	//
	delete file;
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// WebsForm ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

WebsForm::WebsForm(char *formName, WebsFormCb fn) : MaEgiForm(formName)
{
	goFormCallback = fn;
}

////////////////////////////////////////////////////////////////////////////////

WebsForm::~WebsForm()
{
}

////////////////////////////////////////////////////////////////////////////////

void WebsForm::run(MaRequest *rq, char *script, char *uri, 
	char *query, char *postData, int postLen)
{
	int		code;

	//parsePostData(postData, postLen);
	globalPostData = postData;
	globalPostLen = postLen;

	//
	//	GoAhead GoForms write their own headers
	//
	rq->setHeaderFlags(MPR_HTTP_HEADER_WRITTEN, 0);
#if BLD_FEATURE_KEEP_ALIVE
	rq->setNoKeepAlive();
#endif

	(*goFormCallback)((webs_t) rq, (char*) uri, (char*) query);

	//
	//	User must call websDone or websRedirect in the goform to complete
	//	the request. However, websRedirect does not close the request in
	//	Appweb. If websRedirect has not been called, then we set a flag
	//	to cause EGI to keep the request open.
	//
	code = rq->getResponseCode();
	if (300 < code && code <= 399) {
		//
		//	We also need to output headers if redirect is called.
		//

		rq->setHeaderFlags(0, MPR_HTTP_HEADER_WRITTEN);

	} else {
		//
		//	This will stop EGI from closing the socket
		//
		rq->setFlags(MPR_HTTP_DONT_FINISH, -1);
	}
}

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// C APIs ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
extern "C" {

char *strlower(char *string)
{
	char	*s;

	a_assert(string);

	if (string == NULL) {
		return NULL;
	}

	s = string;
	while (*s) {
		if (isupper(*s)) {
			*s = (char) tolower(*s);
		}
		s++;
	}
	*s = '\0';
	return string;
}

////////////////////////////////////////////////////////////////////////////////

char *strupper(char *string)
{
	char	*s;

	a_assert(string);
	if (string == NULL) {
		return NULL;
	}

	s = string;
	while (*s) {
		if (islower(*s)) {
			*s = (char) toupper(*s);
		}
		s++;
	}
	*s = '\0';
	return string;
}

////////////////////////////////////////////////////////////////////////////////

value_t valueInteger(long value)
{
	value_t	v;

	memset(&v, 0x0, sizeof(v));
	v.valid = 1;
	v.type = integer;
	v.value.integer = value;
	return v;
}

////////////////////////////////////////////////////////////////////////////////

value_t valueString(char* value, int flags)
{
	value_t	v;

	memset(&v, 0x0, sizeof(v));
	v.valid = 1;
	v.type = string;
	if (flags & VALUE_ALLOCATE) {
		v.allocated = 1;
		v.value.string = bstrdup(B_L, value);
	} else {
		v.allocated = 0;
		v.value.string = value;
	}
	return v;
}

////////////////////////////////////////////////////////////////////////////////

int emfSchedCallback(int delay, emfSchedProc *proc, void *arg)
{
	new MprTimer(delay, (MprTimerProc) proc, (void*) arg, 
		MPR_TIMER_AUTO_RESCHED);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void emfUnschedCallback(int id)
{
	MprTimer	*timer;

	timer = (MprTimer*) id;
	timer->stop(1000);
	timer->dispose();
}

////////////////////////////////////////////////////////////////////////////////

void emfReschedCallback(int id, int delay)
{
	MprTimer	*timer;

	timer = (MprTimer*) id;
	timer->reschedule(delay);
}

////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// SymHashEntry /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

SymHashEntry::SymHashEntry(char *key, value_t *vp) : MprHashEntry(key)
{
	sym.forw = 0;
	sym.name.value.string = mprStrdup(key);
	sym.name.type = string;
	sym.arg = 0;

	sym.content = *vp;
	if (vp->allocated) {
		sym.content.value.string = mprStrdup(vp->value.string);
	}
}

////////////////////////////////////////////////////////////////////////////////

SymHashEntry::~SymHashEntry()
{
	if (sym.content.allocated) {
		mprFree(sym.content.value.string);
	}
	mprFree(sym.name.value.string);
}

////////////////////////////////////////////////////////////////////////////////

sym_fd_t symOpen(int tableSize)
{
	return (sym_fd_t) new MprHashTable();
}

////////////////////////////////////////////////////////////////////////////////

void symClose(sym_fd_t sd)
{
	MprHashTable	*table = (MprHashTable*) sd;

	delete table;
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symLookup(sym_fd_t sd, char *name)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->lookup(name);
	if (sp == 0) {
		return 0;
	}
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symEnter(sym_fd_t sd, char *name, value_t v, int arg)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = new SymHashEntry(name, &v);
	table->insert(sp);
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////

int symDelete(sym_fd_t sd, char *name)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->lookup(name);
	if (sp) {
		table->remove(sp);
		delete sp;
		return 0;
	} else {
		return -1;
	}
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symFirstEx(sym_fd_t sd, void **current)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->getFirst();
	if (sp == 0) {
		return 0;
	}
	* ((SymHashEntry**) current) = sp;
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////

sym_t *symNextEx(sym_fd_t sd, void **current)
{
	MprHashTable	*table = (MprHashTable*) sd;
	SymHashEntry	*sp;

	sp = (SymHashEntry*) table->getNext((SymHashEntry*) *current);
	if (sp == 0) {
		return 0;
	}
	* ((SymHashEntry**) current) = sp;
	return &sp->sym;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////  User Management //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//	At the momemnt we are using the GoAhead WebServer UM module. This code
//	replaces that with a wrapper over the Appweb security model.
//
#if UNUSED

int umOpen() 
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void umClose() 
{
}

////////////////////////////////////////////////////////////////////////////////

int umRestore(char *filename)
{
	//	FUTURE -- can now implement
#if FUTURE
	auth->resetUserGroup();
	auth->readGroupFile(server, auth, filename);
	auth->readUserFile(server, auth, filename);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umCommit(char *filename)
{
#if FUTURE
	auth->save(filename);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umAddGroup(char *group, short privilege, accessMeth_t am, 
	bool_t protect, bool_t disabled)
{
#if FUTURE
	//
	//	protected == delete protected. This should be done in the UI anyway.
	//	accessMethod == noAuth, basic, digest
	//	disabled == enable the user
	//
	auth->createGroup(group);
	
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umAddUser(char *user, char *password, char *group, bool_t protect, 
	bool_t disabled)
{
#if FUTURE
	//	not supporting disabled, protect
	auth->addUserPassword(user, password);
	auth->addUsersToGroup(group, user);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umDeleteGroup(char *group)
{
#if FUTURE
	auth->removeGroup(group);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umDeleteUser(char *user)
{
#if FUTURE
	auth->removeUser(user);
#endif
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetFirstGroup()
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetNextGroup(char *lastUser)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetFirstUser()
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetNextUser(char *lastUser)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

accessMeth_t umGetGroupAccessMethod(char *group)
{
	return AM_NONE;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umGetGroupEnabled(char *group)
{
	return 1;
}

////////////////////////////////////////////////////////////////////////////////

short umGetGroupPrivilege(char *group)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umGetUserEnabled(char *user)
{
	return 1;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetUserGroup(char *user)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *umGetUserPassword(char *user)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umGroupExists(char *group)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetGroupAccessMethod(char *group, accessMeth_t am)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetGroupEnabled(char *group, bool_t enabled)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetGroupPrivilege(char *group, short privileges)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetUserEnabled(char *user, bool_t enabled)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetUserGroup(char *user, char *password)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int umSetUserPassword(char *user, char *password)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

bool_t umUserExists(char *user)
{
	return 0;
}

#endif // UNUSED 

////////////////////////////////////////////////////////////////////////////////

int websAspDefine(char *name, WebsAspCb fn)
{
	ejsDefineStringCFunction(-1, name, (MprStringCFunction) fn, 0, 
		MPR_VAR_ALT_HANDLE);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void websDecodeUrl(char *decoded, char *token, int len)
{
	maUrlDecode(decoded, len, token, 1, 1);
}

////////////////////////////////////////////////////////////////////////////////

void websDone(webs_t wp, int code)
{
	bool	closeSocket;

	closeSocket = 1;
	((MaRequest*) wp)->finishRequest(code, closeSocket);
}

////////////////////////////////////////////////////////////////////////////////

void websError(webs_t wp, int code, char *msg, ...)
{
	va_list		ap;
	char		buf[MPR_MAX_STRING];

	va_start(ap, msg);
	mprVsprintf(buf, sizeof(buf), msg, ap);
	((MaRequest*) wp)->requestError(code, buf);
	va_end(ap);
}

////////////////////////////////////////////////////////////////////////////////

char *websErrorMsg(int code)
{
	return (char*) maGetHttpErrorMsg(code);
}

////////////////////////////////////////////////////////////////////////////////

void websFooter(webs_t wp)
{
	((MaRequest*) wp)->write("</html>\n");
}

////////////////////////////////////////////////////////////////////////////////

int websFormDefine(char *name, WebsFormCb fn)
{
	char	nameBuf[MPR_HTTP_MAX_URL];

	if (maxForm >= MPR_HTTP_MAX_GO_FORM) {
		mprError(MPR_L, MPR_LOG, "Too many goForms");
		return -1;
	}
	mprSprintf(nameBuf, sizeof(nameBuf), "/goform/%s", name);
	websForms[maxForm++] = new WebsForm(nameBuf, fn);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

char *websGetDateString(websStatType *sbuf)
{
	if (sbuf) {
		MprFileInfo		info;
		info.mtime = sbuf->mtime;
		return (char*) maGetDateString(&info);
	} else {
		return (char*) maGetDateString(0);
	}
}

////////////////////////////////////////////////////////////////////////////////

char *websGetRequestLpath(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getFileName();
}

////////////////////////////////////////////////////////////////////////////////
//
//	Returns the decoded Uri path portion without query
//
char *websGetPath(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getUri();
}

////////////////////////////////////////////////////////////////////////////////

char *websGetUserName(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getUser();
}

////////////////////////////////////////////////////////////////////////////////
//
//	Returns a raw url un-decoded with query
//

char *websGetUrl(webs_t wp)
{
	return (char*) ((MaRequest*) wp)->getOriginalUri();
}

////////////////////////////////////////////////////////////////////////////////

char *websGetVar(webs_t wp, char *var, char *def)
{
	MaRequest	*rq;
	char		*value;

	rq = (MaRequest*) wp;

	if ((value = rq->getVar(MA_FORM_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_HEADERS_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_REQUEST_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_SERVER_OBJ, var, 0))  != 0) {
		return value;
	}
	if ((value = rq->getVar(MA_GLOBAL_OBJ, var, 0))  != 0) {
		return value;
	}
	return def;
}

////////////////////////////////////////////////////////////////////////////////

void websSetVar(webs_t wp, char *var, char *value)
{
	MaRequest	*rq;

	rq = (MaRequest*) wp;
	rq->setVar(MA_GLOBAL_OBJ, var, value);
}

////////////////////////////////////////////////////////////////////////////////

char_t *websGetRequestIpaddr(webs_t wp)
{
	MaRequest	*rq;

	rq = (MaRequest*) wp;
	return rq->getRemoteIpAddr();
}

////////////////////////////////////////////////////////////////////////////////

void websHeader(webs_t wp)
{
	MaRequest	*rq = (MaRequest*) wp;

	rq->write("HTTP/1.0 200 OK\r\n");
	rq->writeFmt("Server: %s\r\n", MPR_HTTP_SERVER_NAME);
	rq->write("Pragma: no-cache\r\n");
	rq->write("Cache-control: no-cache\r\n");
	rq->write("Content-Type: text/html\r\n");
	rq->write("\r\n");
	rq->write("<html>\r\n");
}

////////////////////////////////////////////////////////////////////////////////

int websPageOpen(webs_t wp, char *fileName, char *uri, int mode, int perm)
{
	return file->open(uri, mode, perm);
}

////////////////////////////////////////////////////////////////////////////////

int websPageClose(webs_t wp)
{
	file->close();
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int websPageStat(webs_t wp, char *fileName, char *uri, websStatType* sbuf)
{
	MaRequest		*rq;
	MprFileInfo		info;
 
	rq = (MaRequest*) wp;
	if (rq->host->server->fileSystem->stat(uri, &info) < 0) {
		return MPR_ERR_CANT_ACCESS;
	}
	sbuf->size = info.size;
	sbuf->isDir = info.isDir;
	sbuf->mtime = (time_t) info.mtime;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

void websRedirect(webs_t wp, char *url)
{
	((MaRequest*) wp)->redirect(301, url);
	// ((MaRequest*) wp)->flushOutput(MPR_HTTP_FOREGROUND_FLUSH, 0);
}

////////////////////////////////////////////////////////////////////////////////

void websSetRealm(char *realmName)
{
	MaDir		*dir;
	MaAuth		*auth;

	dir = defaultHost->findBestDir(defaultServer->getServerRoot());

	if (dir == 0) {
		mprError(MPR_L, MPR_LOG,
			"websSetRealm Error: Server not yet configured");
		return;
	}

	auth = dir->getAuth();
	mprAssert(auth);
	if (auth == 0) {
		mprError(MPR_L, MPR_LOG,
			"webSetRealm Error: Server not yet configured");
		return;
	}

	auth->setRealm(realmName);
}

////////////////////////////////////////////////////////////////////////////////

void websSetRequestLpath(webs_t wp, char *fileName)
{
	((MaRequest*) wp)->setFileName(fileName);
}

////////////////////////////////////////////////////////////////////////////////

int websUrlHandlerDefine(char *urlPrefix, char *webDir, int arg, 
	int (*fn)(webs_t wp, char *urlPrefix, char *webDir, int arg, 
	char *url, char *path, char *query), int flags)
{
	mprError(MPR_L, MPR_LOG, "websUrlHandlerDefine is not supported");
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int websValid(webs_t wp)
{
	//
	//	Always return valid. Hope this is sufficient
	//
	return 1;
}

////////////////////////////////////////////////////////////////////////////////

int websValidateUrl(webs_t wp, char *path)
{
	if (maValidateUri(path) == 0) {
		return -1;
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int websWrite(webs_t wp, char* fmt, ...)
{
	va_list		ap;
	char		buf[MPR_MAX_STRING];

	va_start(ap, fmt);
	mprVsprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);

	return ((MaRequest*) wp)->write(buf);
}

////////////////////////////////////////////////////////////////////////////////

int websWriteBlock(webs_t wp, char *buf, int nChars)
{
	return ((MaRequest*) wp)->write(buf, nChars);
}

////////////////////////////////////////////////////////////////////////////////
#if UNUSED_AND_INCOMPLETE

int websAspRequest(webs_t wp, char *localPath)
{
	MaRequest	*rq = (MaRequest*) wp;
	MprFileInfo	*info;
	char		*fileName, *errMsg;
	char		*docBuf;
	int			size;

	info = maGetRequestDocumentInfo(rq);

	size = info->size * sizeof(char);
	docBuf = (char*) mprMalloc(size + 1);
	docBuf[size] = '\0';

	if (maReadDoc(rq, docBuf, size) != size) {
		rq->requestError(404, "Can't read error document");
		mprFree(docBuf);
		return -1;
	}

	fileName = maGetRequestFileName();

	if (espProcessRequest(espRequest, fileName, docBuf, &errMsg) < 0) {
		if (espHandlerService->getFlags() & ESP_FLAGS_ERRORS_TO_BROWSER) {
			rq->writeFmt("<h2>ESP Error in \"%s\"</h2>\n", rq->getUriPath());
			rq->writeFmt("<p>In file: \"%s\"</p>\n", fileName);
			rq->writeFmt("<h3><pre>%s</pre></h3>\n", errMsg);
			rq->writeFmt("<p>To prevent errors being displayed in the "
				"browser, Put <b>\"EspErrors log\"</b> in the "
				"config file.</p>");
			mprFree(errMsg);
		} else {
			rq->requestError(404, 
				"Error processing ESP request %s\n: %s", fileName, 
				errMsg ? errMsg : "");
			if (errMsg) {
				mprFree(errMsg);
			}
			mprFree(docBuf);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}
	}
}

#endif // UNUSED
////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// Socket APIs ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void socketAcceptWrapper(void *data, MprSocket *sock, char *ip, 
	int port, MprSocket *listenSock, int isMprPoolThread)
{
	socketAccept_t	callback;

	mprAssert(sock);

	//
	//	We stored the user's accept callback function in the socket accept data.
	//
	sock->getAcceptCallback(0, (void**) &callback);

	(callback)((int) sock, ip, port, (int) listenSock);
}

////////////////////////////////////////////////////////////////////////////////

int socketOpenConnection(char *host, int port, socketAccept_t accept, int flags)
{
	MprSocket	*sock;
	int			rc, socketFlags;

	socketFlags = 0;
	if (flags & SOCKET_BROADCAST) {
		socketFlags |= MPR_SOCKET_BROADCAST;
	} else if (flags & SOCKET_DATAGRAM) {
		socketFlags |= MPR_SOCKET_DATAGRAM;
	} else if (flags & SOCKET_BLOCK) {
		socketFlags |= MPR_SOCKET_BLOCK;
	}

	sock = new MprSocket();
	mprAssert(sock);

	if (host == 0) {
		//
		//	We store the user's accept function in the socket accept data.
		//
		rc = sock->openServer(host, port, socketAcceptWrapper, (void*) accept, 
			socketFlags);
	} else {
		rc = sock->openClient(host, port, flags);
	}
	if (rc < 0) {
		return -1;
	}
	return (int) sock;
}

////////////////////////////////////////////////////////////////////////////////

void socketCloseConnection(int sid)
{
	MprSocket	*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	sock->close(MPR_SHUTDOWN_BOTH);
	sock->dispose();
}

////////////////////////////////////////////////////////////////////////////////

static void socketIoWrapper(void *data, MprSocket *sock, int socketMask, int
	isMprPoolThread)
{
	socketHandler_t		handler;
	int					mask, arg;

	mprAssert(sock);

	sock->getCallback(0, (void**) &handler, (void**) &arg, &mask);
	(handler)((int) sock, mask, (int) data);
}

////////////////////////////////////////////////////////////////////////////////

void socketCreateHandler(int sid, int mask, socketHandler_t handler, int arg)
{
	MprSocket		*sock;
	int			socketMask;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	socketMask = 0;
	if (mask & SOCKET_READABLE) {
		socketMask |= MPR_SOCKET_READABLE;
	}
	if (mask & SOCKET_WRITABLE) {
		socketMask |= MPR_SOCKET_WRITABLE;
	}
	if (mask & SOCKET_EXCEPTION) {
		socketMask |= MPR_SOCKET_WRITABLE;
	}
	sock->setCallback(socketIoWrapper, (void*) handler, (void*) arg, 
		socketMask);
}

////////////////////////////////////////////////////////////////////////////////

void socketDeleteHandler(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	sock->setCallback(0, 0, 0, 0);
}

////////////////////////////////////////////////////////////////////////////////

int socketEof(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->getEof() ? 1 : 0;
}

////////////////////////////////////////////////////////////////////////////////

int socketFlush(int sid)
{
	return 0;
}

////////////////////////////////////////////////////////////////////////////////

int socketGetHandle(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->getFd();
}

////////////////////////////////////////////////////////////////////////////////

int socketRead(int sid, char *buf, int len)
{
	MprSocket		*sock;

	mprAssert(buf);
	mprAssert(len > 0);

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->read(buf, len);
}

////////////////////////////////////////////////////////////////////////////////

int socketWrite(int sid, char *buf, int len)
{
	MprSocket		*sock;

	mprAssert(buf);
	mprAssert(len > 0);

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->write(buf, len);
}

////////////////////////////////////////////////////////////////////////////////

int socketWriteString(int sid, char *buf)
{
	int		len;

	len = strlen(buf);
	return socketWrite(sid, buf, len);
}

////////////////////////////////////////////////////////////////////////////////

int socketSetBlock(int sid, int flags)
{
	MprSocket		*sock;
	int				old;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	old = sock->getBlockingMode() ? 1 : 0;
	sock->setBlockingMode(flags != 0 ? true : false);
	return old;
}

////////////////////////////////////////////////////////////////////////////////

int socketGetBlock(int sid)
{
	MprSocket		*sock;

	sock = (MprSocket*) sid;
	mprAssert(sock);

	return sock->getBlockingMode() ? 1 : 0;
}

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// Misc ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//	Trace formats normally do not have new lines. 
//
void trace(int level, char *fmt, ...) 
{
	char	buf[MPR_MAX_LOG_STRING];
	va_list args;
	int		len;

	va_start(args, fmt);

	//
	//	mprVsprintf always ensures there is a null terminator. Subtract one
	//	for a new line
	//
	len = mprVsprintf(buf, sizeof(buf) - 1, fmt, args);
	buf[len++] = '\n';
	buf[len] = '\0';

	mprLog(level, buf);
	va_end(args);
}

////////////////////////////////////////////////////////////////////////////////

void traceRaw(char *buf) 
{
	mprLog(0, buf);
}

////////////////////////////////////////////////////////////////////////////////
//
//	Allocate a new file handle.  On the first call, the caller must set the
//	handle map to be a pointer to a null pointer.  *map points to the second
//	element in the handle array.
//

int hAlloc(void ***map)
{
	int		*mp;
	int		handle, len, memsize, incr;

	a_assert(map);

	if (*map == NULL) {
		incr = H_INCR;
		memsize = (incr + H_OFFSET) * sizeof(void**);
		if ((mp = (int*) balloc(B_L, memsize)) == NULL) {
			return -1;
		}
		memset(mp, 0, memsize);
		mp[H_LEN] = incr;
		mp[H_USED] = 0;
		*map = (void**) &mp[H_OFFSET];

	} else {
		mp = &((*(int**)map)[-H_OFFSET]);
	}

	len = mp[H_LEN];

	//
	//	Find the first null handle
	//
	if (mp[H_USED] < mp[H_LEN]) {
		for (handle = 0; handle < len; handle++) {
			if (mp[handle+H_OFFSET] == 0) {
				mp[H_USED]++;
				return handle;
			}
		}
	} else {
		handle = len;
	}

	//
	//	No free handle so grow the handle list. Grow list in chunks of H_INCR.
	//
	len += H_INCR;
	memsize = (len + H_OFFSET) * sizeof(void**);
	if ((mp = (int*) brealloc(B_L, (void*) mp, memsize)) == NULL) {
		return -1;
	}
	*map = (void**) &mp[H_OFFSET];
	mp[H_LEN] = len;
	memset(&mp[H_OFFSET + len - H_INCR], 0, sizeof(int*) * H_INCR);
	mp[H_USED]++;
	return handle;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Free a handle.  This function returns the value of the largest
//	handle in use plus 1, to be saved as a max value.
//

int hFree(void ***map, int handle)
{
	int		*mp;
	int		len;

	a_assert(map);
	mp = &((*(int**)map)[-H_OFFSET]);
	a_assert(mp[H_LEN] >= H_INCR);

	a_assert(mp[handle + H_OFFSET]);
	a_assert(mp[H_USED]);
	mp[handle + H_OFFSET] = 0;
	if (--(mp[H_USED]) == 0) {
		bfree(B_L, (void*) mp);
		*map = NULL;
	}

	//
	//	Find the greatest handle number in use.
	//
	if (*map == NULL) {
		handle = -1;
	} else {
		len = mp[H_LEN];
		if (mp[H_USED] < mp[H_LEN]) {
			for (handle = len - 1; handle >= 0; handle--) {
				if (mp[handle + H_OFFSET])
					break;
			}
		} else {
			handle = len;
		}
	}
	return handle + 1;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Allocate an entry in the halloc array.
//

int hAllocEntry(void ***list, int *max, int size)
{
	char	*cp;
	int		id;

	a_assert(list);
	a_assert(max);

	if ((id = hAlloc((void***) list)) < 0) {
		return -1;
	}

	if (size > 0) {
		if ((cp = (char*) balloc(B_L, size)) == NULL) {
			hFree(list, id);
			return -1;
		}
		a_assert(cp);
		memset(cp, 0, size);

		(*list)[id] = (void*) cp;
	}

	if (id >= *max) {
		*max = id + 1;
	}
	return id;
}

////////////////////////////////////////////////////////////////////////////////

int fmtAlloc(char **buf, int maxSize, char *fmt, ...)
{
	va_list		args;
	int			rc;

	va_start(args, fmt);	
	rc = mprAllocVsprintf(buf, maxSize, fmt, args);
	va_end(args);	
	return rc;
}

////////////////////////////////////////////////////////////////////////////////

int fmtValloc(char **buf, int maxSize, char *fmt, va_list arg)
{
	return mprAllocVsprintf(buf, maxSize, fmt, arg);
}

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////// Security Handler //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//	The GaCompatHandler is created for each incoming HTTP request. We tell
//	the Handler base class that we must run ALWAYS as a non-terminal
//	handler. Handlers be non-terminal where they can modify the request,
//	but not actually handle it.
//

GaCompatHandler::GaCompatHandler(char *extensions) :
	MaHandler("gaCompatHandler", extensions, MPR_HANDLER_ALWAYS)
{
	if (defaultServer == 0) {
		defaultServer = maGetHttp()->findServer("default");
		defaultHost = defaultServer->getDefaultHost();
	}
#if UNUSED
	if (defaultInterp == 0) {
		defaultInterp = ejsGetDefaultInterp();
	}
	if (defaultServer == 0 || defaultHost == 0 || defaultInterp == 0) {
		error(MPR_L, "Undefined server, host or EJS engine");
		return;
	}
#endif
}


////////////////////////////////////////////////////////////////////////////////
//
//	Destructor for the GaCompatHandler
//

GaCompatHandler::~GaCompatHandler()
{
}


////////////////////////////////////////////////////////////////////////////////
//
//	For maximum speed in servicing requests, we clone a pre-existing
//	handler when an incoming request arrives.
//

MaHandler *GaCompatHandler::cloneHandler()
{
	return new GaCompatHandler(extensions);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Called to see if this handler should process this request.
//	Return TRUE if you want this handler to match this request
//

int GaCompatHandler::matchRequest(MaRequest *rq, char *uri, int uriLen)
{
	//	Always match
	return 1;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Run the handler to service the request
//

int GaCompatHandler::run(MaRequest *rq)
{
	char  *uri;

	uri = rq->getUri();

	if (securityChecks(rq) < 0) {
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	}
	return 0;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Not yet debugged
//

int GaCompatHandler::securityChecks(MaRequest *rq)
{
#if FUTURE
	accessMeth_t	am;
	MaAuth			*auth;
	char			*type, *accessLimit, *requiredPassword;
	char			*uriPath, *ext, *mimeType, *query, *password, *user;
	char			*realm;
	int				flags, nRet, isSecure, authType;

	accessLimit = 0;
	realm = "UNKNOWN";

	auth = rq->getAuth();
	uriPath = maGetRequestUriPath(rq);
	ext = maGetRequestUriExt(rq);
	mimeType = maGetRequestMimeType(rq);
	query = maGetRequestUriQuery(rq);
	isSecure = maIsRequestSecure(rq);

	//
 	//	Get the critical request details
 	//
	password = maGetRequestPassword(rq);
	user = maGetRequestUserName(rq);

	//
	//	Get the access limit for the URL.  Exit if none found.
	//
	accessLimit = umGetAccessLimit((char*) uriPath);
	if (accessLimit == NULL) {
		return 0;
	}
		 
	//
	//	Check to see if URL must be encrypted
	//
#if BLD_FEATURE_SSL_MODULE
	nRet = umGetAccessLimitSecure(accessLimit);
	if (nRet && isSecure) {
		maSetResponseError(rq, 405, 
			"Access Denied\nSecure access is required.");
		trace(3, "SEC: Non-secure access attempted on <%s>\n", uriPath);
		bfree(B_L, accessLimit);
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	}
#endif

	//
	//	Get the access limit for the URL
	//
	am = umGetAccessMethodForURL(accessLimit);
	authType = (am == AM_DIGEST) ? MPR_HTTP_AUTH_DIGEST : MPR_HTTP_AUTH_BASIC;

	if (strcmp("127.0.0.1", maGetRequestIpAddr(rq)) == 0) {
		//
		//	Local access is always allowed
		//
		return 0;
	}

	if (am == AM_NONE) {
		//
		//	URL is supposed to be hidden!  Make like it wasn't found.
		//
		maSetResponseError(rq, 404, "Page Not Found");
		bfreeSafe(B_L, accessLimit);
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	} 

	requiredPassword = umGetUserPassword((char*) user);
	auth->setRequiredPassword();

	if (user && *user) {

		if (! umUserExists((char*) user)) {
			formatAuthResponse(rq, auth, realm, authType, 401, 
				"Access Denied\nUnknown User");
			bfreeSafe(B_L, accessLimit);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}
		if (! umUserCanAccessURL((char*) user, accessLimit)) {
			formatAuthResponse(rq, auth, realm, authType, 403, 
				"Access Denied\nProhibited User");
			bfreeSafe(B_L, accessLimit);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}
		if (password && * password) {

			auth->setRequiredPassword();
			if (requiredPassword) {
				if (gstrcmp(password, requiredPassword) != 0) {
					maSetResponseError(rq, 401, 
						"Access Denied\nWrong Password");
					bfreeSafe(B_L, accessLimit);
					return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
				}
				bfree (B_L, requiredPassword);
			}

#if BLD_FEATURE_DIGEST
		} else if (mprStrcmpAnyCase(maGetRequestAuthType(rq), "digest") == 0) {

			char *digestCalc;

			//
			//	Check digest for equivalence
			//
			requiredPassword = umGetUserPassword((char*) user);
			maSetRequestPassword(requiredPassword);
#if USING_AUTH_HANDLER_FOR_THIS
			a_assert(wp->digest);
			a_assert(wp->nonce);
							 
			digestCalc = websCalcDigest(wp);
			maCalcDigest(&requiredDigest, userName, requiredPassword, realm, 
				uri, nonce, qop, nc, 
				cnonce, rq->getMethod());
			mprFree(requiredPassword);
			a_assert(digestCalc);

			if (gstrcmp(wp->digest, digestCalc) != 0) {
				formatAuthResponse(rq, auth, realm, authType, 401, 
					"Access Denied\nWrong Password");
				bfreeSafe(B_L, accessLimit);
				return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
			}
			bfree (B_L, digestCalc);
#endif
#endif
		} else {
			//
			//	No password has been specified
			//
			formatAuthResponse(rq, auth, realm, authType, 401, 
				"Access to this document requires a password");
			bfreeSafe(B_L, accessLimit);
			return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
		}

	} else if (am != AM_FULL) {
		//
		//	This will cause the browser to display a password / username dialog
		//
		formatAuthResponse(rq, auth, realm, authType, 401, 
			"Access to this document requires a User ID");
		bfreeSafe(B_L, accessLimit);
		return MPR_HTTP_HANDLER_FINISHED_PROCESSING;
	}
#endif
	return 0;
}


////////////////////////////////////////////////////////////////////////////////

void GaCompatHandler::formatAuthResponse(MaRequest *rq, MaAuth *auth, 
	char *realm, int authType, int code, char *userMsg)
{
#if BLD_FEATURE_DIGEST
	char		*nonceStr;
	char		*etag;
#endif
	char		*buf, *headers;

	rq->incAccessError();

	mprLog(3, "auth: formatAuthResponse: code %d, %s\n", code, userMsg);
	mprAllocSprintf(&buf, MPR_HTTP_BUFSIZE, 
		"<HTML><HEAD>\n<TITLE>Authentication Error: %s</TITLE>\n</HEAD>\r\n"
		"<BODY><H2>Authentication Error: %s</H2></BODY>\n</HTML>\r\n",
		userMsg, userMsg);

	if (authType == MPR_HTTP_AUTH_BASIC) {
		mprAllocSprintf(&headers, MPR_MAX_STRING,
			"WWW-Authenticate: Basic realm=\"%s\"", realm);

#if BLD_FEATURE_DIGEST
	} else if (authType == MPR_HTTP_AUTH_DIGEST) {

		//
		//	Use the etag as our opaque string
		//
		etag = rq->getEtag();
		if (etag == 0) {
			etag = "";
		}
		maCalcNonce(&nonceStr, rq->host->getSecret(), etag, realm);

		mprAllocSprintf(&headers, MPR_MAX_STRING,
			"WWW-Authenticate: Digest realm=\"%s\", domain=\"%s\", "
			"qop=\"auth\", nonce=\"%s\", opaque=\"%s\", algorithm=\"MD5\", "
			"stale=\"FALSE\"", 
			realm, rq->host->getName(), nonceStr, etag);

		mprFree(nonceStr);
#endif
	}
	rq->setHeader(headers);
	rq->formatAltResponse(code, buf, MPR_HTTP_DONT_ESCAPE);
	rq->flushOutput(MPR_HTTP_FOREGROUND_FLUSH, MPR_HTTP_FINISH_REQUEST);
	mprFree(headers);
	mprFree(buf);
}

////////////////////////////////////////////////////////////////////////////////

}	// extern "C"
#else
void mprCompatModuleDummy() {}

#endif // BLD_FEATURE_GACOMPAT_MODULE

//
// Local variables:
// tab-width: 4
// c-basic-offset: 4
// End:
// vim: sw=4 ts=4 
//
