#include <linux/module.h>
#include <linux/miscdevice.h>
#include <linux/watchdog.h>
#include <linux/fs.h>
#include <linux/delay.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/param.h>
#include <linux/notifier.h>
#include <linux/reboot.h>
#include <asm/irq.h>
#include <asm/traps.h>
#include <asm/delay.h>
#include <linux/ioctl.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <mach/gpio.h>

#define DEVICE_NAME					"GPIO_SOUND"
//gong: change MAJOR from 14 to 15
#define GPIO_SOUND_MAJOR            15
#define GPIO_SOUND_MINOR			120
#define GPIO_SOUND_ON                    1
#define GPIO_SOUND_OFF                   0

#define SOUND_IOC_MAGIC    0xB1
#define SOUND_ON     _IO(SOUND_IOC_MAGIC, 0)
#define SOUND_OFF     _IO(SOUND_IOC_MAGIC, 1)

static beep_state = GPIO_SOUND_OFF;
/***************************************************************************
  PB17 is the gpio pin to control the sound, high enable; low disable
****************************************************************************/
static int gpio_sound_open(struct inode *inode, struct file *file);
static int gpio_soundi_release(struct inode *inode, struct file *file);
static ssize_t gpio_sound_write(struct file *filep, const char *buffer, size_t length,
                loff_t *off);
static int gpio_sound_ioctl(struct inode *inode,struct file *filp,
			  unsigned int cmd, unsigned long arg);

/*  fixed for 2.4 kernel, owner was ifdef'ed out for 2.0 kernel */
static struct file_operations gpio_sound_ops = {
        owner:          THIS_MODULE,
        write:          gpio_sound_write,
        ioctl:          gpio_sound_ioctl,
        open:           gpio_sound_open,
        release:        gpio_soundi_release  /* a.k.a. close */
};

static int gpio_sound_open(struct inode *inode, struct file *file)
{
	return 0;
}

static int gpio_soundi_release(struct inode *inode, struct file *file)
{
	return 0;
}

static ssize_t gpio_sound_write(struct file *filep, const char *buffer, size_t length,
                loff_t *off)
{
	int i;

	if(beep_state == GPIO_SOUND_ON)
	    return 0;

	for(i = 0; i < length; i++)
	{
		gpio_direction_output(AT91_PIN_PB17,1);
		udelay(2000);//gong: 1000->2000
		udelay(2000);//gong: 1000->2000
		udelay(2000);//gong: 1000->2000
		udelay(2000);//gong: 1000->2000
	}
	
	gpio_direction_output(AT91_PIN_PB17,0);	
	return length;
}
static int gpio_sound_ioctl(struct inode *inode,struct file *filp,
			  unsigned int cmd, unsigned long arg)
{
    switch(cmd)
    {
    case GPIO_SOUND_ON:
		gpio_direction_output(AT91_PIN_PB17,1);
		beep_state = GPIO_SOUND_ON;
        break;
    case GPIO_SOUND_OFF:
    	gpio_direction_output(AT91_PIN_PB17,0);
    	beep_state = GPIO_SOUND_OFF;
        break;
    default:
        break;            
    }
    return 0;
}
/***************************************************************************/
static int __init gpio_sound_init(void)
{
	int ret;
	
	if ((ret = register_chrdev(GPIO_SOUND_MAJOR, DEVICE_NAME, &gpio_sound_ops) < 0)) {
		printk ("%s device failed with %d\n",DEVICE_NAME, ret);
		return(ret);
	}

	at91_set_GPIO_periph(AT91_PIN_PB17, 0);
	at91_set_deglitch(AT91_PIN_PB17, 1);
	gpio_direction_output(AT91_PIN_PB17,0);
	udelay(10);
	printk("gpio_sound init ok!\n");
	
	return 0;
}

/***************************************************************************/

static void __exit gpio_sound_exit(void)
{
	unregister_chrdev(GPIO_SOUND_MAJOR, DEVICE_NAME);
}

/***************************************************************************/

module_init(gpio_sound_init);
module_exit(gpio_sound_exit);

MODULE_AUTHOR("Gong, Chao");
MODULE_DESCRIPTION("UltraKey Touch GPIO_Sound Driver");
MODULE_LICENSE("GPL");
