/*
 * cy8c214 - driver for cy8c214 touch pad chips
 *
 * Copyright (C) 2009 Honeywell
 * Copyright (C) 2009 Gong Chao
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/device.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <linux/workqueue.h>
#include <linux/suspend.h>
#include <linux/debugfs.h>
#include <linux/seq_file.h>
#include <linux/mutex.h>
#include <linux/miscdevice.h>
#include <linux/io.h>
#include <asm/uaccess.h>
#include <mach/gpio.h>

#include "cy8c214.h"

void __iomem  *base;
static int resetflag=0;
//static unsigned long counter=0;

//#define REG_SLIDER_OUT_FLAG		(CS2BASEADD+0x0000e)
//#define REG_SLIDER_SIG_OUT		(CS2BASEADD+0x00010)

//#define I2C_M_STOP		0x80
//#define I2C_M_WRITE_NOACK		0x100

//int i2c_netcore_master_send(struct i2c_client *client,const char *buf ,int count, u16 extra_flag);
//int i2c_netcore_master_recv(struct i2c_client *client, char *buf ,int count, u16 extra_flag);

static int cy8c214_attach_adapter(struct i2c_adapter *adapter);
static int cy8c214_detach_client(struct i2c_client *client);
static int cy8c214_detect(struct i2c_adapter *adapter, int address, int kind);
static int Slider_Read(char *buffer, int size);

static struct i2c_driver cy8c214Slider_driver = {
	.driver = {
		.name	= "cy8c214",
	},
	.id		= I2C_DRIVERID_CY8C214,
	.attach_adapter	= cy8c214_attach_adapter,
	.detach_client	= cy8c214_detach_client,
};

/*
 * We keep a copy of the client device found, since we are really only
 * need one device for the misc device interface.
 */
static struct i2c_client *client;

static unsigned short ignore[] = { I2C_CLIENT_END };
static unsigned short normal_addr[] = { 0x04, I2C_CLIENT_END };

inline void Slider_Reset(void)
{
#if 0
	volatile unsigned short reg; 
	reg = readw(REG_SLIDER_OUT_FLAG);
	writew(reg | 0x01, REG_SLIDER_OUT_FLAG);
	
	reg = readw(REG_SLIDER_SIG_OUT);
	writew(reg | 0x02, REG_SLIDER_SIG_OUT);
	mdelay(10);
	writew(reg & 0xFFFFD, REG_SLIDER_SIG_OUT);
	mdelay(100);
	printk("Slider fpga reset\n");
#endif
	gpio_direction_output(AT91_PIN_PB15,1);
	mdelay(50);
	gpio_direction_output(AT91_PIN_PB15,0);
	mdelay(200);
	printk("Slider reset\n");
}



static struct i2c_client_address_data addr_data = {
	.normal_i2c = normal_addr,
	.probe = ignore,
	.ignore = ignore,
};

/*
 * Real code
 */
static int cy8c214_attach_adapter(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter, &addr_data, cy8c214_detect);
	//int aaa=i2c_probe(adapter, &addr_data, cy8c214_detect);
	//cy8c214_detect(adapter,4,-1);
	//return aaa;
}

static int cy8c214_detach_client(struct i2c_client *client)
{
	int err;

	if ((err = i2c_detach_client(client)))
		return err;

	kfree(i2c_get_clientdata(client));
	return 0;
}

/* This function is called by i2c_probe */
static int cy8c214_detect(struct i2c_adapter *adapter, int address, int kind)
{
	struct i2c_client *new_client;
#if 1
	struct cy8c214_data *data;
	int err = 0;

	/* OK. For now, we presume we have a valid client. We now create the
	   client structure, even though we cannot fill it completely yet. */
	if (!(data = kzalloc(sizeof(struct cy8c214_data), GFP_KERNEL))) {
		err = -ENOMEM;
		goto exit;
	}
	
	client = new_client = &data->client;
	i2c_set_clientdata(new_client, data);
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &cy8c214Slider_driver;
	new_client->flags = 0;

	/* Now, we would do the remaining detection. But the PCF8591 is plainly
	   impossible to detect! Stupid chip. */

	/* Determine the chip type - only one kind supported! */
	/*if (kind <= 0)
		kind = pcf8591;*/

	/* Fill in the remaining client fields and put it into the global 
	   list */
	strlcpy(new_client->name, "cy8c214", I2C_NAME_SIZE);
	
	/* Tell the I2C layer a new client has arrived */
	if ((err = i2c_attach_client(new_client)))
		goto exit_kfree;
	/* Initialize the PCF8591 chip */
	//pcf8591_init_client(new_client);

#if 0
	/* Register sysfs hooks */
	device_create_file(&new_client->dev, &dev_attr_out0_enable);
	device_create_file(&new_client->dev, &dev_attr_out0_output);
	device_create_file(&new_client->dev, &dev_attr_in0_input);
	device_create_file(&new_client->dev, &dev_attr_in1_input);

	/* Register input2 if not in "two differential inputs" mode */
	if (input_mode != 3 )
		device_create_file(&new_client->dev, &dev_attr_in2_input);
		
	/* Register input3 only in "four single ended inputs" mode */
	if (input_mode == 0)
		device_create_file(&new_client->dev, &dev_attr_in3_input);
#endif	
	//slide_test();
	return 0;
	
	/* OK, this is not exactly good programming practice, usually. But it is
	   very code-efficient in this case. */

exit_kfree:
	kfree(data);
exit:
	return err;
#else
	return 0;
#endif
}

static int cy8c214_open(struct inode * inodep, struct file * fp)
{
	//unsigned char array[16];
	//int i;
#if 0	
	__iic_sequential_read_psoc(8, 0, 5, array);
	for(i = 0; i < 5; i++)
		printk(" %x", array[i]);
	printk("\n");
#endif
	return 0;
}

static int cy8c214_release(struct inode * inodep, struct file *fp)
{
	return 0;
}

static int cy8c214_ioctl(struct inode *inodep, struct file * fp, unsigned int cmd, unsigned long arg)
{
	int j;
	static unsigned char value[16];

	j = 200;
	switch(cmd) {
		case SLIDER_IOCRESET:
			Slider_Reset();
			break;
		case SLIDER_IOCGSTATUS:
		#if 1
			if(Slider_Read(value, 7) < 0)
				return -1;
			//iic_sequential_read_psoc_3(8, 0, 5, value);
			copy_to_user((char __user *)arg, value, 7);
		#else
			//slide_test();
		#endif	
			break;
	}
	return 0;
}

static int i2c_netcore_master_send(struct i2c_client *client,const char *buf ,int
		count, u16 extra_flag)
{
	int ret;
	struct i2c_adapter *adap=client->adapter;
	struct i2c_msg msg;

	msg.addr = client->addr;
	msg.flags = client->flags & I2C_M_TEN;
	msg.flags |= extra_flag;
	msg.len = count;
	msg.buf = (char *)buf;

	ret = i2c_transfer(adap, &msg, 1);
	return (ret == 1) ? count : ret;
}

static int i2c_netcore_master_recv(struct i2c_client *client, char *buf ,int count, u16
		extra_flag)
{
	struct i2c_adapter *adap=client->adapter;
	struct i2c_msg msg;
	int ret;
	msg.addr = client->addr;
	msg.flags = client->flags & I2C_M_TEN;
	msg.flags |= I2C_M_RD;
	msg.flags |= extra_flag;
	msg.len = count;
	msg.buf = buf;
	ret = i2c_transfer(adap, &msg, 1);
	//printk("return value: %d\n",ret); //gong:
	return (ret == 1) ? count : ret;
}

static int Slider_Read(char *buffer, int size)
{
	unsigned short slaveaddr = 0;
#if 0
	if(i2c_netcore_master_send(client, &slaveaddr, 2, 0) != 2)
	{
		Slider_Reset(); //gong:
		printk("Send Slider device address wrong");
		return -1;
	}
#endif	
	int temp=i2c_netcore_master_recv(client, buffer, size, 0);
	if(temp != size)
	{
		if(temp!=-1)
		{
//			unsigned long hoho;
			if(1==resetflag)
			{
				Slider_Reset();
			}

			//printk("i2c bus timeout error, reset the master!\n");
			//printk("error counter: %d\n",counter++);
			__raw_writel(0x80, base+0x0);
			mdelay(20);
			__raw_writel(0x24, base+0x0);
			__raw_writel(0x41000, base+0x4);
			__raw_writel(0x7a7a, base+0x10);
			udelay(100);	
			resetflag=1;
#if 0	
			hoho=__raw_readl(base+0x0);
			printk("*********************2nd\n");
			printk("CR: 0x%x \n",hoho);
			hoho=__raw_readl(base+0x4);
			printk("MMR: 0x%x \n",hoho);
			hoho=__raw_readl(base+0x10);
			printk("CWGR: 0x%x \n",hoho);
			hoho=__raw_readl(base+0x20);
			printk("SR: 0x%x \n",hoho);
			hoho=__raw_readl(base+0x30);
			printk("RHR: 0x%x \n",hoho);
#endif
		}
		else
			resetflag=0;
	//	printk("Read i2c data wrong\n");
		return -1;
	}
	else
	{
		if(buffer[0] == 0xff && 
			buffer[1] == 0xff &&
			buffer[2] == 0xff &&
			buffer[3] == 0xff &&
			buffer[4] == 0xff &&
			buffer[5] == 0xff &&
			buffer[6] == 0xff)
		{
			printk("slider data wrong\n");
			Slider_Reset(); //gong:
			resetflag=0;
			return -1;
		}
	}
	resetflag=0;	
	return size;
}

static struct file_operations cy8c214_fops = {
	.owner		= THIS_MODULE,
	.open		= cy8c214_open,
	.ioctl		= cy8c214_ioctl,
	.release	= cy8c214_release,
};

static struct miscdevice cy8c214_miscdrv = {
	.minor		= PC110PAD_MINOR,
	.name		= "slider",
	.fops		= &cy8c214_fops,
};

static int __init cy8c214_init(void)
{
	int rc;
	base=ioremap(0xfff88000, 0x00004000);
	at91_set_GPIO_periph(AT91_PIN_PB15, 1); 
	at91_set_deglitch(AT91_PIN_PB15, 1);
	gpio_direction_output(AT91_PIN_PB15,1);
	mdelay(50);
	gpio_direction_output(AT91_PIN_PB15,0);
	mdelay(150);
	if ((rc = i2c_add_driver(&cy8c214Slider_driver)) < 0)
		return rc;
		
	if ((rc = misc_register(&cy8c214_miscdrv)) < 0) {
		i2c_del_driver(&cy8c214Slider_driver);
		return rc;
	}
	printk("init slider driver ok\n");
	return 0;
}

static void __exit cy8c214_exit(void)
{
	misc_deregister(&cy8c214_miscdrv);
	i2c_del_driver(&cy8c214Slider_driver);
}

//MODULE_AUTHOR("Henry, Xin <Xin.YuePeng@honeywell.com>");
MODULE_AUTHOR("Gong Chao <Chao.Gong@honeywell.com>");
MODULE_DESCRIPTION("CY8C214 Slider driver");
MODULE_LICENSE("GPL");

module_init(cy8c214_init);
module_exit(cy8c214_exit);
