#include <linux/init.h>
#include <linux/clk.h>
#include <linux/err.h>
#include <linux/io.h>
#include <linux/interrupt.h>
#include <linux/atmel_pwm.h>
#include <mach/gpio.h>
#include <linux/module.h>
#include <linux/miscdevice.h>
#include <linux/watchdog.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/param.h>
#include <linux/notifier.h>
#include <linux/reboot.h>
#include <linux/cdev.h>
#include <asm/irq.h>
#include <asm/traps.h>
#include <asm/delay.h>
#include <asm/uaccess.h>
#include <linux/delay.h>
#include "ultrakey_pwm.h"

void __iomem  *base;
static struct cdev ultrakey_pwm_cdev;

static int ultrakey_pwm_sound_open(struct inode *inode, struct file *file);
static int  ultrakey_pwm_release(struct inode *inode, struct file *file);
static ssize_t ultrakey_pwm_write(struct file *filep, const char *buffer, size_t length,
                loff_t *off);
static int ultrakey_pwm_ioctl(struct inode *inode, struct file *filp,
			unsigned int command, unsigned long arg);
			
/*  fixed for 2.4 kernel, owner was ifdef'ed out for 2.0 kernel */
static struct file_operations ultrakey_pwm_ops = {
        owner:          THIS_MODULE,
        ioctl:          ultrakey_pwm_ioctl,
        open:           ultrakey_pwm_sound_open,
        release:        ultrakey_pwm_release  /* a.k.a. close */
};

static int ultrakey_pwm_sound_open(struct inode *inode, struct file *file)
{
	return 0;
}

static int  ultrakey_pwm_release(struct inode *inode, struct file *file)
{
	return 0;
}

static int ultrakey_pwm_ioctl(struct inode *inode, struct file *filp,
			unsigned int command, unsigned long arg)
{
	u32 m = iminor(inode);
	int retval = 0;
	base=ioremap(PWMC_BASE, 0x00004000);
	switch(m) {
		case PWM0_MINOR:
			switch(command) {
				case PWM_IOCSWR:
					__raw_writel((arg%MAX_PWSIZE), base+0x210);
					break;
				case PWM_IOCGWR:
					{
						retval = __put_user( (__raw_readl(base+0x204)), (int __user *)arg);		
					}
					break;
				default:
					break;
			}
			break;
#if 0
		case PWM1_MINOR:
			switch(command) {
				case PWM_IOCSWR:
					port = (PWM_PWR1+MCF_MBAR);
					*(port) = ((arg%MAX_PWSIZE)<<24);
					break;
				case PWM_IOCGWR:
					{
						port = (PWM_PWR1+MCF_MBAR);
						retval = __put_user( ((*(port))>>24), (int __user *)arg);		
						port = (PWM_CR1+MCF_MBAR);
						printk("Ctl reg: 0x%x\n",		*(port));		
					}
					break;
				default:
					break;
			}
			break;
		case PWM2_MINOR:
			break;
#endif
		default:
			break;
	}
	
	return retval;
}


/***************************************************************************/
static int __init ultrakey_pwm_init(void)
{
	int ret;
	dev_t	dev_id;
	
	dev_id = MKDEV(ULTRAKEY_PWM_MAJOR, ULTRAKEY_PWM_MINOR);
	ret = register_chrdev_region(dev_id, ULTRAKEY_PWM_COUNT, DEVICE_NAME);
	if(ret) {
		printk("Register ultrakey pwm device failed\n");
		return -1;
	}
	cdev_init(&ultrakey_pwm_cdev, &ultrakey_pwm_ops);
	cdev_add(&ultrakey_pwm_cdev, dev_id, ULTRAKEY_PWM_COUNT);
	
	//Init the PWM0
	at91_set_B_periph(AT91_PIN_PB7, 1);
	
	/* enable the pwm clk */
	base=ioremap(PMC_BASE, 256);
	__raw_writel(1<<20, base+0x10);
	
	/* enable pwm0 */
	base=ioremap(PWMC_BASE, 0x00004000);
	__raw_writel(1<<0, base+0x8);
	__raw_writel(1<<9 | DEFAULT_CLK, base+0x200);
	__raw_writel(0x9000, base+0x204);
	__raw_writel(0xffff, base+0x208);
	__raw_writel(1<<0, base+0x4);

	printk(KERN_ALERT "init pwm ok\n");

//	mdelay(2000);
//	__raw_writel(0xffff, base+0x210);

	return 0;
}

/***************************************************************************/

static void __exit ultrakey_pwm_exit(void)
{
	dev_t dev_id = MKDEV(ULTRAKEY_PWM_MAJOR, ULTRAKEY_PWM_MINOR);

	cdev_del(&ultrakey_pwm_cdev);
	unregister_chrdev_region(dev_id, ULTRAKEY_PWM_COUNT);
	printk(KERN_ALERT "rmmod pwm ok\n");
}

module_init(ultrakey_pwm_init);
module_exit(ultrakey_pwm_exit);

MODULE_AUTHOR("Gong, Chao");
MODULE_DESCRIPTION("Ultrakey PWM simple Driver");
MODULE_LICENSE("GPL");
