#include <linux/init.h>
#include <linux/clk.h>
#include <linux/err.h>
#include <linux/io.h>
#include <linux/cdev.h>
#include <linux/module.h>
#include <linux/miscdevice.h>
#include <linux/watchdog.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/param.h>
#include <linux/notifier.h>
#include <linux/reboot.h>
#include <linux/delay.h>
#include <asm/traps.h>
#include <asm/delay.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <mach/gpio.h>
#include "uart_mode_ctrl.h"

#define DEVICE_NAME								"SerialSwitcher"
#define SERIALSWITCHER_MAJOR		6
#define SERIALSWITCHER_MINOR		240

static int PortA_Mode = MODE_RS232;
static int PortB_Mode = MODE_RS485_R;
static int PortC_Mode = MODE_RS485_R;

void __iomem  *base;

/****************************************************************************/
static int serial_mode_switcher_open(struct inode *inode, struct file *file);
static int serial_mode_switcher_release(struct inode *inode, struct file *file);
static ssize_t serial_mode_switcher_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
                unsigned long arg);

/*  fixed for 2.4 kernel, owner was ifdef'ed out for 2.0 kernel */
static struct file_operations serial_mode_switcher_ops = {
        owner:			THIS_MODULE,
        ioctl:				serial_mode_switcher_ioctl,
        open:				serial_mode_switcher_open,
        release:			serial_mode_switcher_release  /* a.k.a. close */
};

static int serial_mode_switcher_open(struct inode *inode, struct file *file)
{
	return 0;
}

static int serial_mode_switcher_release(struct inode *inode, struct file *file)
{
	return 0;
}

static ssize_t serial_mode_switcher_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
                unsigned long arg)
{
    int serial_no;
	if(cmd == SERIAL_LED_REGISTER)
	{
		printk("This Function not support in Ultrakey Touch\n");
		return -1;
	}
	else
	    serial_no = (int) arg;
	
	switch(serial_no) 
	{
		case UARTA_FLAG:
			switch(cmd)
			{
				case MCFSERIAL_IOCS232M:
				case MCFSERIAL_IOCS485MR:
				case MCFSERIAL_IOCS485MT:
				case MCFSERIAL_IOCS422M:
					//printk("ttyS1 only work on RS232 mode!\n");
					printk("UARTA not support on UltrakeyTouch!\n");
					break;
				default:
					break;
			}
			break;
		
		case UARTD_FLAG:
			switch(cmd)
			{
				case MCFSERIAL_IOCS232M:
					printk("ttyS2 only work on RS485 or RS422 mode\n");
					break;
				case MCFSERIAL_IOCS485MR:
					udelay(600);
					at91_set_B_periph(AT91_PIN_PD5, 0);
					gpio_direction_output(AT91_PIN_PD6,1);
					udelay(100);
					PortB_Mode = MODE_RS485_R;
					printk("ttyS2 now work on RS485 mode\n");
					break;
				case MCFSERIAL_IOCS485MT:
					udelay(600);
					at91_set_B_periph(AT91_PIN_PD5, 0);
					gpio_direction_output(AT91_PIN_PD6,1);
					udelay(100);
					PortB_Mode = MODE_RS485_T;
					printk("ttyS2 now work on RS485 mode\n");
					break;
				case MCFSERIAL_IOCS422M:
					udelay(600);
					at91_set_GPIO_periph(AT91_PIN_PD5, 0);
					at91_set_deglitch(AT91_PIN_PD5, 1);
					gpio_direction_output(AT91_PIN_PD5,0);

					gpio_direction_output(AT91_PIN_PD6,0);
					udelay(100);
					printk("ttyS2 now work on RS422 mode, cannot use ttyS3!\n");
					PortB_Mode = MODE_RS422;
					break;
				default:
					break;
			}
			break;
		
		case UARTC_FLAG:
				switch(cmd)
				{
					case MCFSERIAL_IOCS422M:
					case MCFSERIAL_IOCS485MR:
					case MCFSERIAL_IOCS485MT:
					case MCFSERIAL_IOCS232M:
					//	printk("ttyS3 only work on RS485 mode!\n");
						printk("UARTC not support on UltrakeyTouch!\n");
						break;
					default:
						break;
				}
			break;

		case UARTB_FLAG:
				switch(cmd)
				{
					case MCFSERIAL_IOCS422M:
					case MCFSERIAL_IOCS485MR:
					case MCFSERIAL_IOCS485MT:
					case MCFSERIAL_IOCS232M:
					//	printk("ttyS3 only work on RS485 mode!\n");
						printk("UARTB not support on UltrakeyTouch!\n");
						break;
					default:
						break;
				}
			break;

		default:
			break;
	}
	
	if(cmd == MCFSERIAL_IOCGMOD) 
	{
		printk("This Function not support in Ultrakey Touch\n");
		return -1;
	}	
    
	switch(cmd)
        {
        case TX_LED_ON:
        	gpio_direction_output(AT91_PIN_PD9,0);
			break;
        case TX_LED_OFF:
        	gpio_direction_output(AT91_PIN_PD9,1);
			break;
        case RX_LED_ON:
			gpio_direction_output(AT91_PIN_PD8,0);
	        //printk("rx led on\n");
        	break;
        case RX_LED_OFF:
			gpio_direction_output(AT91_PIN_PD8,1);
        	break;
        default:
        	break;
        }

	return 0;
}

/***************************************************************************/
static int __init serial_mode_switcher_init(void)
{
	int ret;
	
	if ((ret = register_chrdev(SERIALSWITCHER_MAJOR, DEVICE_NAME, &serial_mode_switcher_ops) < 0)) {
		printk ("%s device failed with %d\n",DEVICE_NAME, ret);
		return(ret);
	}
	
	/* set UART2 to RS485 mode */
//	unsigned int hoho;
	base=ioremap(USART2_BASE, 0x00004000);
//	hoho=__raw_readl(base+0x4);
//	printk("old mode: %x \n",hoho);
	__raw_writel(0xc00008c1, base+0x4);
//	udelay(100);
//	hoho=__raw_readl(base+0x4);
//	printk("new mode: %x \n",hoho);

	/* set UART1 to RS485 mode */
	base=ioremap(USART1_BASE, 0x00004000);	
	__raw_writel(0xc00008c1, base+0x4);
	at91_set_GPIO_periph(AT91_PIN_PD6, 1);
	at91_set_deglitch(AT91_PIN_PD6, 1);
	gpio_direction_output(AT91_PIN_PD6,1);
	printk("ttyS1 now work on RS232 mode!\nttyS2 and ttyS3 now work on RS485 mode!\n");

	/* init LED */
	at91_set_GPIO_periph(AT91_PIN_PD8, 1);
	at91_set_deglitch(AT91_PIN_PD8, 1);
	gpio_direction_output(AT91_PIN_PD8,1);//RX led
	at91_set_GPIO_periph(AT91_PIN_PD9, 1);
	at91_set_deglitch(AT91_PIN_PD9, 1);
	gpio_direction_output(AT91_PIN_PD9,1);//TX led

	return 0;
}

/***************************************************************************/

static void __exit serial_mode_switcher_exit(void)
{
	unregister_chrdev(SERIALSWITCHER_MAJOR, DEVICE_NAME);
}

/***************************************************************************/

module_init(serial_mode_switcher_init);
module_exit(serial_mode_switcher_exit);

MODULE_AUTHOR("Gong, Chao");
MODULE_DESCRIPTION("UltrakeyTouch UART mode control Driver");
MODULE_LICENSE("GPL");
